$TITLE:'COVARIANCE RTN'
$SUBTITLE:'INITIALIZE'
      SUBROUTINE ICOVAR ( N, WORK, ISTAT )
C
C         INITIALIZE THE COVARIANCE ROUTINES WORK AREA
C
C    INPUT:
C              N         I*4       NUMBER OF ELEMENTS EACH POINT
C    OUTPUT:
C              WORK()    R*8       WORK ARRAY
C                                  Length >  3 + N + N*N
C              ISTAT     I*4       0 FINE, -1 PROBLEM
C                                  === LIMITS TO 15 PARAMETERS ===
C
C    WORK USE
C              1         FLAG      =12345 CORRECT
C              2         N         NUMBER OF ELEMENTS IN POINTS
C              3         NPTS      RUNNING SUM OF NUMBER OF POINTS
C              4->4+N-1  SUMS OF VALUES
C              () NxN    SUMS OF Xi * Xj
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION     WORK(*)
C
C         ---------------------------------------------------
C
      IST  = -1
      IF ( N.LE.0 )  GOTO 900
      IF ( N.GT.15 ) GOTO 900
C
      WORK(1)  = 12345.0D0
C
      WORK(2)  = N
C
      WORK(3)  = 0.0D0
C
      IXVEC    =  4
      IXMTRX   =  4 + N
C
      DO   200 II =  1, N
C
          WORK ( IXVEC ) = 0.0D0
          IXVEC = IXVEC + 1
C
          DO  100 I = 1, N
             WORK ( IXMTRX ) = 0.0D0
             IXMTRX  = IXMTRX + 1
100       CONTINUE
C
200   CONTINUE
C
      IST  = 0
C
900   CONTINUE
C
      ISTAT  = IST
      RETURN
      END
$PAGE
$SUBTITLE:'ADD VALUE, A POINT'
      SUBROUTINE PCOVAR ( XVEC, WORK, ISTAT )
C
C         ADD IN A POINT TO THE COVARIANCE WORK ARRAY
C         -- MUST HAVE INTIALZED WORK WITH COVARI --
C
C    INPUT:
C              XVEC(N)   R*8       THE N VALUES AT THIS POINT
C
C    INPUT/OUTPUT:
C              WORK()    R*8       WORK ARRAY
C                                  Length >  3 + N + N*N
C    OUTPUT
C              ISTAT     I*4       0 FINE, -1 PROBLEM
C
C    WORK USE
C              1         FLAG      =12345 CORRECT
C              2         N         NUMBER OF ELEMENTS IN POINTS
C              3         NPTS      RUNNING SUM OF NUMBER OF POINTS
C              4->4+N-1  SUMS OF VALUES
C              () NxN    SUMS OF Xi * Xj
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION     XVEC(*)
      DIMENSION     WORK(*)
C
C              ---------------------------------------------
C
      IST  = -1
C
      W1   =  WORK(1)
      W2   =  WORK(2)
      W3   =  WORK(3)
C
      IF ( W1.NE.12345.0D0 ) GOTO 900
      N    =  W2
      IF ( N.LE.0 .OR. N.GT.15 ) GOTO 900
C
      NPT  = W3
      IF ( NPT.LT.0 ) GOTO 900
C
C                   -------------
C
      IXSUM   =  4
      IXSQ    =  4 + N
C
      NPT     =  NPT + 1
C
      DO  200 II = 1, N
C
          XII          = XVEC(II)
          WORK(IXSUM)  = WORK(IXSUM) + XII
          IXSUM        = IXSUM + 1
C
          DO  100  I = 1, N
C
               XI          = XVEC(I)
               WORK(IXSQ)  = WORK(IXSQ)  + XI*XII
               IXSQ        = IXSQ + 1
C
100       CONTINUE
C
200   CONTINUE
C
      WORK(3)  =  NPT
      IST      =  0
C
900   CONTINUE
C
      ISTAT    =  IST
C
      RETURN
      END
$PAGE
$SUBTITLE:'CURRENT RESULTS'
      SUBROUTINE RCOVAR ( NPTS,XAVG,XSD,CORR,WORK, ISTAT )
C
C         RETURN THE CURRENT RESULTS FOR A COVARIANCE COMPUTATION
C         -- MUST HAVE INTIALZED WORK WITH COVARI --
C
C    INPUT:
C              WORK()    R*8       WORK ARRAY
C                                  Length >  3 + N + N*N
C    OUTPUT
C              NPTS      I*4       NUMBER POINTS SUMMED
C              XAVG()    R*8       AVERAGES
C              XSD()     R*8       STANDARD DEVIATIONS
C              CORR(,)   R*8       CORRELATION COEF
C
C              ISTAT     I*4       0 FINE, <0 PROBLEM
C                                  -1 NOT INITIALIZED
C                                  -2 NOT ENOUGH POINTS
C
C    WORK USE
C              1         FLAG      =12345 CORRECT
C              2         N         NUMBER OF ELEMENTS IN POINTS
C              3         NPTS      RUNNING SUM OF NUMBER OF POINTS
C              4->4+N-1  SUMS OF VALUES
C              () NxN    SUMS OF Xi * Xj
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION     XAVG(*),XSD(*),CORR(*)
      DIMENSION     WORK(*)
C
C              ---------------------------------------------
C
      IST  = -1
C
      W1   =  WORK(1)
      W2   =  WORK(2)
      W3   =  WORK(3)
C
      IF ( W1.NE.12345.0D0 ) GOTO 900
      N    =  W2
      IF ( N.LE.0 .OR. N.GT.15 ) GOTO 900
C
      NPT  = W3
      IF ( NPT.LT.0 ) GOTO 900
C
C              DEFAULT RETURNS
C
      NPTS = NPT      
C
      DO   200 II=1, N
         XAVG(II) = 0.0D0
         XSD(II)  = 0.0D0
         DO  100 I = 1, N
           ITO         = (II-1)*N + I
           CORR(ITO)  = 0.0D0
100     CONTINUE
200   CONTINUE
C
      IF ( NPT.LT.3 ) THEN
         IST  = -2
         GOTO 900
      ENDIF
C
C
C
C                   -------------
C
      FNPT   =  NPT
      FNM1   =  NPT - 1
      RATIO  =  FNM1/FNPT
C
C           COMPUTE AVERAGE AND STANDARD DEVIATIONS
C
      DO  300  IX   = 1, N
C
         IXSUM       =  IX + 3
         IXDG        =  (IX - 1)*N + IX + 3 + N
C
         SUM         =  WORK (IXSUM)
         SUMSQ       =  WORK (IXDG)
         XBAR        =  SUM /  FNPT   
         VAR         =  ( SUMSQ - FNPT*XBAR*XBAR)/FNM1
C
         IF (VAR.LT.0.0D0) VAR = 0.0D0
C
         XAVG(IX)    =  XBAR
         XSD (IX)    =  SQRT(VAR)
C
300   CONTINUE
C
C         COMPUTE THE CORRELATION MATRIX
C
      DO  500  II = 1, N
C
          DO  400  I = 1, N
C
             AVI       =  XAVG(I)
             AVII      =  XAVG(II)
             SDI       =  XSD(I)
             SDII      =  XSD(II)
C
             DIV       =  RATIO * SDI * SDII
             DIVU      =  DIV
             IF (ABS(DIVU).LT.1.0D-8)  DIVU = 1.0D0
C
             IXSQ      =  (II-1)*N + I + N + 3
C
             XYSUM     =  WORK (IXSQ)
C
             AVSQ      =  XYSUM/FNPT  - AVI*AVII
             COEF      =  AVSQ/DIVU
C
             IF ( ABS(COEF) .LT.1.0D-8 .AND. I.EQ.II) COEF = 1.0D0
C
             ITO        =  (II-1)*N + I
C
             CORR(ITO) =  COEF
C
C
400       CONTINUE
500   CONTINUE
C
      IST  =  0
C
900   CONTINUE
C
      ISTAT    =  IST
C
      RETURN
      END
