$TITLE:'REAL SYMETRIC MATRIX EIGENVALUES'
      SUBROUTINE EVRSMX (A,NI,NPI,D,V,NROT)
C
C      EIGENVALUES AND EIGENVECTORS OF REAL-SYMETRIC-MATRIX
C
C      BASED ON 'JACOBI' PRESS ET AL PG 346 PLUS
C      BASED IN TURN ON ESIPACK AND HANDBOOK..WILKINSON + REINSCH
C
C      INPUT: 
C
C      A    REAL SYMETRIC MATRIX  ( NP * NP )
C      N    DIMENSION OF A ( LOGICAL )
C      NP   PHYSICAL DIMENSION OF A  ( NP .LE. N OF COURSE )
C
C      OUTPUT:
C
C      D    VECTOR NP LONG, 1ST N = EIGENVALUES ( ARBRITARY ORDER )
C      V    MATRIX OF EIGEN VECTORS   ( NP *  NP )
C      NROT NUMBER OF MAJOR ROTATIONS IN SOLUTION
C           =  -1    INVALID INPUT ( N OR NP TOO LARGE, INCONSISTANT )
C           =  -2    VERY BAD CONVERGENCE, OVER 50 CYCLES  AND NOT 
C                    DONE
C                    ( ONE CYCLE = N*(N-1) ROTATIONS IF ALL ROTATIONS
C                      PERFORMED. MAY DO ALMOST FULL CYCLES TILL LAST.
C                      TYPICALLY ON WELL BEHAVED MATRIX, 3 CYCLES )
C
C
C      8-5-87  JRC ONTO MICRO
C              MADE ALL EXTERNAL INTEGERS = I*4
C                       INTERNAL INTEGERS = I*2 FOR SPEED
C                   ALL REALS  = REAL*8  ( AND CHANGED ROUTINE NAMES )
C              REDUCED MAX SIZE TO 15 
C              ADDED USE OF NROT NEGATIVE VALUES AS ERROR FLAGS
C              AUTOMATICLY CALLED SORT OF E-VALUE/VECTORS ON GOOD EXIT
C
C   =======================================================================
C    ----N.B.   PRESS ET AL SAY PG 348 THAT UNDERFLOWS CAN OCCUR
C               AND THAT THEY ASSUME JUST SET THESE TO ZERO.  ON
C               THIS AND MOST MACHINES, UNDEFLOW = ABORT
C   =======================================================================
C
C
      PARAMETER (NMAX=15)
C
      IMPLICIT DOUBLE PRECISION ( A-H, O-Z )
      IMPLICIT INTEGER*2          ( I-N )
C
      INTEGER*4   NI, NPI, NROT
C
C
      DIMENSION A(NPI,NPI),D(NPI),V(NPI,NPI)
C
      DIMENSION B(NMAX),Z(NMAX)
C
C
C
C      ERROR/CONSISTANT VALUES
C
       IVAL = -1
       IF ( NPI.GT.NMAX ) GOTO 800
       IF (  NI.GT.NPI  ) GOTO 800
C
       IVAL = 0
C
       N  =  NI
       NP =  NPI
C
C   ----------------------------------------------------------------
C       INITIALIZE TO IDENITY  ---  TOP OF OPERATIONAL CODE
C   ----------------------------------------------------------------
C
      DO 12 IP=1,N
        DO 11 IQ=1,N
          V(IP,IQ)=0.0D0
11      CONTINUE
        V(IP,IP)=1.0D0
12    CONTINUE
C
C     ------------------------------------
C     INITIALIZE B AND D TO DIAGNOL OF A
C     ------------------------------------
C
      DO 13 IP=1,N
        B(IP)=A(IP,IP)
        D(IP)=B(IP)
        Z(IP)=0.0D0
13    CONTINUE
C
C     ==============================================
C     INITIALIZE NROT, BEGIN MAJOR LOOP = HERE TO 24
C     ==============================================
C
      NROT=0
C
      DO 24 I=1,50
C
C       SUM UP OFF DIAGNOL ELEMENTS
C
        SM=0.0D0
        DO 15 IP=1,N-1
          DO 14 IQ=IP+1,N
            SM=SM+DABS(A(IP,IQ))
14        CONTINUE
15      CONTINUE
C
C       ------------------------------
C       CONVERGENCE/END TEST HERE  ---
C       NORMAL END TO 900          ---
C       ------------------------------
C
        IF(SM.EQ.0.0D0)  GOTO 900
C
C           SPECIAL THRESHOLD 1ST 3 SWEEPS
C
        IF(I.LT.4)THEN
          TRESH=0.2D0 * SM/N**2
        ELSE
          TRESH=0.0D0
        ENDIF
C
C       ROTATION LOOP
C
        DO 22 IP=1,N-1
          DO 21 IQ=IP+1,N
C
            G=100.0D0*DABS(A(IP,IQ))
C
C  -------------------------------------------------------
C     SPECIAL CASE ADDED BY PRESS ET AL --SKIP ROTATION IF
C     NOT NEEDED AND AFTER 4 EARLIER ROTATIONS
C   ------------------------------------------------------
C
            IF((I.GT.4).AND.(DABS(D(IP))+G.EQ.DABS(D(IP)))
     *         .AND.(DABS(D(IQ))+G.EQ.DABS(D(IQ))))THEN
              A(IP,IQ)=0.0D0
C
            ELSE IF(DABS(A(IP,IQ)).GT.TRESH)THEN
              H=D(IQ)-D(IP)
              IF(DABS(H)+G.EQ.DABS(H))THEN
                T=A(IP,IQ)/H
C
              ELSE
                THETA=0.5D0 * H/A(IP,IQ)
                T=1.0D0/(DABS(THETA)+DSQRT(1.0D0+THETA**2))
                IF(THETA.LT.0.0D0)T=-T
              ENDIF
C
              C=1.0D0/DSQRT(1+T**2)
              S=T*C
              TAU=S/(1.0D0+C)
              H=T*A(IP,IQ)
              Z(IP)=Z(IP)-H
              Z(IQ)=Z(IQ)+H
              D(IP)=D(IP)-H
              D(IQ)=D(IQ)+H
              A(IP,IQ)=0.0D0
C
C              CASE OF ROTATIONS Q .LE. J LE. P
C
              DO 16 J=1,IP-1
                G=A(J,IP)
                H=A(J,IQ)
                A(J,IP)=G-S*(H+G*TAU)
                A(J,IQ)=H+S*(G-H*TAU)
16            CONTINUE
C
C               CASE OF ROTATIONS P .LT. J .LT. Q
C
              DO 17 J=IP+1,IQ-1
                G=A(IP,J)
                H=A(J,IQ)
                A(IP,J)=G-S*(H+G*TAU)
                A(J,IQ)=H+S*(G-H*TAU)
17            CONTINUE
C
C               CASE OF ROTATIONS Q .LT. J .LE. N
C
              DO 18 J=IQ+1,N
                G=A(IP,J)
                H=A(IQ,J)
                A(IP,J)=G-S*(H+G*TAU)
                A(IQ,J)=H+S*(G-H*TAU)
18            CONTINUE
C
C
              DO 19 J=1,N
                G=V(J,IP)
                H=V(J,IQ)
                V(J,IP)=G-S*(H+G*TAU)
                V(J,IQ)=H+S*(G-H*TAU)
19            CONTINUE
C
C            END OF ONE CYCLE
C
              NROT=NROT+1
            ENDIF
C
21        CONTINUE
C
C
22      CONTINUE
C
C       UPDATE D WITH SUM OF TA(P,Q) AND RE INITIALIZE Z
C
        DO 23 IP=1,N
          B(IP)=B(IP)+Z(IP)
          D(IP)=B(IP)
          Z(IP)=0.0D0
23      CONTINUE
C
24    CONTINUE
C 
C     ------------------   END OF BIG DO LOOP   ----------------
C     
C     IF FALLS THROUGH HAVE NOT CONVERGED  ----
C
      IVAL = -2
      GOTO 800
C
C
C    ERROR EXIT
C
800   CONTINUE
C
      NROT  = IVAL
C
C
C     EXIT
C
900   CONTINUE
C
C     IF GOT GOOD RESULTS, SORT E-VALUES, E-VECTORS 
C
      IF ( NROT.GE.0 ) CALL EVSORT( D,V,NI,NPI )
C
      RETURN
      END
$PAGE
$SUBTITLE:'SORT EIGENVECTORS/VALUES'
      SUBROUTINE EVSORT(D,V,N,NP)
C
C      SORT THE EIGENVECTORS AND VALUES CONPUTED IN
C      THE REAL-SYMETRIC MATRIX EIGENVALUE ROUTIEN  EVRSMX
C
C     INPUT/OUTPUT
C
C          D     = EIGEN VALUE VECTOR N LONG LOGICALLY
C          V     = MARTRIX OF EV'S  ( NP * NP ) PHYSICALLY
C
C     INPUT:
C          N     = LOGICAL LENGTH OF VECTORS
C          NP    = PHYSICAL LENGTH OF VECTORS/MATRIX
C
C       8-5-87 JRC    PUT ON MICRO
C                     NO ESSENTIAL CHANGES FROM PRESS ET AL
C                   
C         ALL INTEGERS = I*4   ( NO SPEED NEEDED )
C         ALL REALS = REAL*8
C
C
      IMPLICIT DOUBLE PRECISION  ( A-H,  O-Z )
      IMPLICIT INTEGER*4           ( I-N )
C
      DIMENSION D(NP),V(NP,NP)
C
      DO 13 I=1,N-1
        K=I
        P=D(I)
C
        DO 11 J=I+1,N
          IF(D(J).GE.P)THEN
            K=J
            P=D(J)
          ENDIF
11      CONTINUE
C
        IF(K.NE.I)THEN
          D(K)=D(I)
          D(I)=P
C
          DO 12 J=1,N
            P=V(J,I)
            V(J,I)=V(J,K)
            V(J,K)=P
12        CONTINUE
C
        ENDIF
C
13    CONTINUE
C
      RETURN
      END
