      SUBROUTINE RFFT ( DATA,NIN,ISIGNI )
C
C        REAL FFT -- R*8 VERSION
C        BASED ON NUMBERICAL RECIEPES
C
C        DATA   =  AN ARRAY OF N ELEMENTS -- A REAL TIME SERIES
C        NIN    =  LENGTH -- MUST BE POWER OF TWO
C        ISIGNI =  1  FORWARD XFORM ( ANY .GT. 0 )
C                 -1  INVERSE       ( ANY .LT. 0 )
C
C       XFORM  OUTPUT
C           COMPLEX ARRAY OF POSITIVE HALF OF SPECTRUM
C           EXCEPT ITEMS 1 AND 2
C
C           NORMALIZE ON FORWARD TRANSFORM 
C
C                 SIN WAVE OF AMPLITUDE 1 COMES OUT WITH 
C                 MAGNITUDE OF 1.
C
C                 1.0 AS MAGNITUDE IN SPECTRUM ON INVERSE
C                 GIVES SINE WAVE OF AMPLITUDE 1.0
C
C
C             EVEN K :  F(K/2)  =  DATA(K-1) = J*DATA(K)
C
C             DC ( K=0 ) LAST VALUE ARE REAL BECAUSE SERIES
C             REAL AND ASSUMED CYCLIC  --- THEREFORE
C
C                      F(0)    =   DATA( 1 )
C                      F(N/2)  =   DATA( 2 )  -- STORED HERE
C
C            ( THIS IS THE COMMON CONVERTION )
C
C         DONE BY LETTING ARRAY LOOK LIKE COMPLEX SIGNAL OF
C         HALF THE LENGTH AND THEN SHUFFLING THE DATA WHICH
C         CAN FIND BY SYMETRY ( SEE PG398+ N.R. )
C
C         INVERSE  SHUFFLE FIRST
C
C   
C
$INCLUDE:'WRDSIZ.INC'
C
      REAL*8      WR,WI,WPR,WPI,WTEMP,THETA
C
      DIMENSION   DATA(*)
C
C
C     ----------------------------------------------------------
C
C      ------------           ---------         --------     ------
C
C         HALF THE LENGTH FOR INTERNAL USE (N.R. ALGORITHM)
C
      N  = NIN / 2
C  
C      ------------           ---------         --------     ------
C
C         SAFETY CODE
C
      ISIGN  = 1
      IF ( ISIGNI.LT.0 ) ISIGN = -1
C
C
C      THE SMALLEST STEP ON UNIT CIRCLE ANGLE
C
      THETA =  6.28318530717959D0 / 2.0D0 / DBLE(N)
C
      C1    =  0.5
C
C         IF XFORM -- DO XFORM FIRST
C
      IF ( ISIGN.EQ.1 )  THEN
C
        C2  =  -0.5
C
C       -------------------------
        CALL CFFT ( DATA, N, +1 )
C       -------------------------
C
      ELSE
C 
C         INVERSE--SET UP FOR UN SHUFFLE - XFORM BELOW
C
        C2     =   0.5
        THETA  =  -THETA
C
      ENDIF
C
C        SET UP FOR DOING (EXP(-J*2*PI*K/N)) BY POWERS
C
      WPR   =  -2.0D0 * DSIN(0.5D0*THETA)**2
      WPI   =           DSIN(THETA)
      WR    =   1.0D0 + WPR
      WI    =   WPI
      N2P3  =   2*N  +  3
C
C           SHUFFLE
C
C
C       (BEGIN AT 2, ITEM 1 BELOW)
C
C
      DO 100 I=2, N/2+1
C
        I1  =   2*I  - 1
        I2  =   I1   + 1
        I3  =   N2P3 - I2
        I4  =   I3   + 1
C
        WRS =    (WR)
        WIS =    (WI)
C
        H1R    =   C1 * ( DATA(I1) + DATA(I3) )
        H1I    =   C1 * ( DATA(I2) - DATA(I4) )
        H2R    =  -C2 * ( DATA(I2) + DATA(I4) )
        H2I    =   C2 * ( DATA(I1) - DATA(I3) )
C
        DATA(I1)   =   H1R + WRS*H2R - WIS*H2I
        DATA(I2)   =   H1I + WRS*H2I + WIS*H2R
        DATA(I3)   =   H1R - WRS*H2R + WIS*H2I
        DATA(I4)   =  -H1I + WRS*H2I + WIS*H2R
C
C           STEP ON UNIT CIRCLE
C
        WTEMP   =  WR
        WR      =  WR*WPR - WI*WPI+WR
        WI      =  WI*WPR + WTEMP*WPI+WI
C
100    CONTINUE
C
C         FORWARD XFORM HANDLE ELEMENTS 1 AND 2
C
      IF ( ISIGN.EQ.1 ) THEN
C
        H1R     =   DATA(1)
        DATA(1) =   H1R + DATA(2)
        DATA(2) =   H1R - DATA(2)
C
      ELSE
C
C         INVERSE XFORM HANDLE ELEMENTS 1 AND 2
C
        H1R     =   DATA(1)
        DATA(1) =   C1 * ( H1R + DATA(2) ) 
        DATA(2) =   C1 * ( H1R - DATA(2) )
C
C         INVERSE XFORM -- DO XFORM
C
C       ------------------------
        CALL CFFT ( DATA, N, -1)
C       ------------------------
C
      ENDIF
C 
      RETURN
      END
$PAGE
      SUBROUTINE CFFT ( DATA,NN,ISIGNI )
C
C      COMPLEX FFT OF NN ELEMENTS OR REAL OF 2*NN
C
C       DATA    INPUT/OUTPUT  ARRAY OF NN COMPLEX ITEMS
C               ( X + J*Y)  IN  K, K+1  ( K ODD )
C
C       NN      NUMBER OF COMPLEX INPUT / OUTPUT ITEMS
C               MUST BE POWER OF TWO
C       ISIGN   DIRECTION  
C               +1    FORWARD XFORM
C               -1    INVERSE XFORM
C
C
$INCLUDE:'WRDSIZ.INC'
C
      REAL*8      WR,WI,WPR,WPI,WTEMP,THETA
      DIMENSION   DATA(*)
C
C        SAFETY CODE FOR POTENTIAL BAD INPUT
C
      ISIGN  = 1
      IF ( ISIGNI .LT.  0 ) ISIGN = -1
C 
C       DO BIT REVERSAL -- RE ORDERING
C
      N  =  2 * NN
      J  =  1
C
      DO 200 I=1,N,2
C
        IF( J.GT.I )  THEN
C
C          SWAP
C
          TEMPR     =  DATA(J)
          TEMPI     =  DATA(J+1)
          DATA(J)   =  DATA(I)
          DATA(J+1) =  DATA(I+1)
          DATA(I)   =  TEMPR
          DATA(I+1) =  TEMPI
C
        ENDIF
C
        M  =  N/2
C
100   CONTINUE
C
        IF ( (M.GE.2) .AND. (J.GT.M) ) THEN
            J  =  J-M
            M  =  M/2
            GO TO 100
        ENDIF
C
        J  =  J + M
C
200    CONTINUE
C
C          MULTIPLES -- BUTTERFLIES
C
C        OUTTER LOOP LOG BASE 2 OF NN  -- NUMBER OF BITS IN NN
C
      MMAX=2
C
300   CONTINUE
C
      IF (  N.GT.MMAX  ) THEN
C
        ISTEP  =  2*MMAX
C
C          STEP ON UNIT CIRCLE
C 
        THETA  =  6.28318530717959D0 / (ISIGN*MMAX)
        WPR    = -2.D0 * DSIN(0.5D0*THETA)**2
        WPI    =  DSIN(THETA)
        WR     =  1.D0
        WI     =  0.D0
C
C
        DO 500 M=1,MMAX,2
C
          DO 400 I=M,N,ISTEP
C
            J       =  I + MMAX
            TEMPR   =  (WR) * DATA(J)   - (WI) * DATA(J+1)
            TEMPI   =  (WR) * DATA(J+1) + (WI) * DATA(J)
C
            DATA(J)    =  DATA(I)   - TEMPR
            DATA(J+1)  =  DATA(I+1) - TEMPI
            DATA(I)    =  DATA(I)   + TEMPR
            DATA(I+1)  =  DATA(I+1) + TEMPI
C
400        CONTINUE
C
          WTEMP   =  WR
          WR      =  WR*WPR - WI*WPI+WR
          WI      =  WI*WPR + WTEMP*WPI+WI
C
500      CONTINUE
C
      MMAX = ISTEP
C
      GO TO 300
C
      ENDIF
C
C        SCALE IF FORWARD X-FORM
C
      IF ( ISIGN.GT.0 ) THEN
C
          FNOR  = 1.0D0 / NN
C
          DO 600 I = 1, N
          DATA(I) = FNOR * DATA(I)
600       CONTINUE
C
      ENDIF
C
C
      RETURN
C
      END
