$TITLE:'LOAD PLM COEFFICIENTS'
      SUBROUTINE LDPCOF ( FNAME, LMAX,LMXLD,A,CCOEF,SCOEF,ISTAT)
C
C
C              LOAD PLM COEFFICIENTS
C
C    MODHIST:
C              2-8-94  JRC CODED
C              2-11    ADDED MODPHASE, FILE
C
C
C         STORAGE ORDER:
C                        [0,0],[1,0],[1,1],[2,0],[2,1]...
C
C                        FOR LMAX:  NSTORE = (N+1)*(N+2)/2
C
C    ROUTINE OPENS FILE AND LOADS
C    ALLOWS COMMENTS AND COMMANDS IN FILE
C
C    AUXILIARY COMMON(S) HAS SOME DATA IF USER WANTS
C         (NAME, SCALE.... )
C
C    WILL RESCALE COEFFICIENTS TO 'NATURAL' OR UN-NORMALIZED
C    VALUES,  FITS WITH ROUTINES  PLM, PLMA, DPLML....
C
C
C
C
C       ==  SUBROUTINE ARGUMENTS DEFINATION ==
C
C    INPUT:
C              FNAME          C*30      FILE NAME
C              LMAX           I*4       MAX SIZE OF INDEX ( L,L )
C     
C    OUTPUT:
C              LMXLD          I*4       MAX DEGREE LOADED
C              A              R*8       SCALE RADIUS OR 1.0 IF NONE
C              CCOEF(.)       R*8       ARRAY OF COS  COEFFICEINTS
C              SCOEF(.)       R*8          "  "  SIN       "
C              ISTAT          I*4       STATUS,  0 OKAY, -1 ERROR PARMS.
C                                                1 FILE ERRORS
C
C
C               =======  FILE  FORMAT  ==========
C
C         FILE CAN HAVE COMMAND BEGINNING IN COL. 1
C              CAN HAVE COMMENT CHARACTER IN COL. 1
C
C      ---------------------------------------------------
C                    DATA THREE FORMATS
C
C         TWO ITEMS / LINE
C
C                 COS-COEF.    SIN-COEF
C
C                 ( SIN-COEF IGNORED FOR M=0, STORED AS ZERO ALWAYS)
C
C          ONE ITEM / LINE
C
C              COS-COEF
C              SIN-COEF  ( OMITTED FOR M=0 )
C                        (ORDER SAME AS STORAGE )
C
C         EXPLICIT L,M
C
C         L    M    COS-COEF  SIN-COEF   or
C         M    L      "         "
C
C         -------------------------------------------
C
C              -----   COMMANDS/COMMENTS  -----
C
C    
C
C    WITHIN FILE CAN BE COMMANDS AND COMMENTS
C
C    IGNORES:
C              BLANK LINES
C              LINE BEGINNING WITH  SEMICOLON ";", ASTRICK "*", 
C                                   LETTER C   "C"
C
C
C                    ====  COMMANDS  =====
C
C              BEGINNING IN COLUMN 1 THE FOLLOWING ARE
C              VALID COMMANDS.  4 CHARACTERS ARE SCANNED FOR
C              COMMAND IDENTIFICATION
C
C  COMMNAD               ARGUMENTS
C  -------               ---------
C    NAME                ALPHABETIC MODEL NAME 1 TO 10 CHARACTERS LONG
C
C    NMAX                LMAX      MAX DEGREE IN FILE
C
C    NORMALIZATION       NONE           (NO NORMALIZATION, NATURAL)
C                        FULL           FULL NORMALATION ( LIKE Ylm )
C                        GEOPHYS        NORMAL PBAR USED  GRAVITY  or
C                        GRAVITY
C                        SCHMIDT        (Mag field types)  or
C                        MAGNETIC
C                        ( Default = none )
C
C    SCALE               VALUE TO MULTIPLY COEFFICIENTS 
C                        ( GM For example )
C
C    RADIUS              Radius         RADIUS TO USE IN A/R 
C                                       (Default 1.0)
C
C
C    FILE                FILENAME Nskip 
C                                       New file to use for input
C                                       Very useful for initialization
C                                       of large data files
C
C                                       Nskip = records to skip, default
C                                               to zero
C    END                                END OF INPUT
C
C  (Data format / ordering commands )
C
C    TWOLINE                            Two items / line m .ne.0
C                                       One item for m = 0
c
C    ONELINE                            One item per line
C
C    LMEXPLICIT     IXL  IXM            The l, m  index's are on the
C                                       line in position IXL, and IXM.
C                                       (usualy 1,2  or 2,1 )
C                                       Assumes C-coef in 3, 
C                                       S-coef in position 4
C
C
C                   -------------------------------------
C
C                             -------------------
C         ** NOTE BENE:       USES LOGICAL UNIT 9    ***
C                             -------------------
C
C
$INCLUDE:'WRDSIZ.INC'
C
$INCLUDE:'LMCOEF.INC'
C
C
      CHARACTER*30   FNAME
      DIMENSION      CCOEF (*),  SCOEF(*)
C
C
$PAGE
C
C
C              INPUT BUFFER -- AND PARSE ROUTINE DEFINATIONS
C
      CHARACTER*80   LINE
C
      CHARACTER*4    CMD
      CHARACTER*1    CMD1
C
      EQUIVALENCE    ( LINE, CMD  )
      EQUIVALENCE    ( CMD,  CMD1 )
C
C         VARIABLES FOR PARSE LINE OUTPUT
C
C              ITYPV = TYPE   0=EMPTY, 1=NUMBER,2=CHARACTER, ELSE=ERROR
C              RVAL  = REAL VALUE WHEN REAL OR LENGHT WHEN CHARACTER
C              SVAL  = CHARACTER STRING WHEN CHARACTER
C
C              NOTE:  TAKES  F,E,OR D FORMATS
C
C                     ANY WHITE SPACE OR COMMA OR COLON OR COMBO 
C                     IS END OF TOKEN
C
C                     FOR CHARACTER, IF MORE THAN ONE TOKEN, ENCLOSE
C                     IN DOUBLE QUOTES ( "ONE AND TWO" = ONE STRING)
C
C
      DIMENSION     ITYPV(10), RVAL(10)
      CHARACTER*80  SVAL(10)
C
      CHARACTER*30   FNIN
      CHARACTER*1    CDUM
C
      DATA   MXPARM  / 10  /
C
C  -------------------------------------------------------------------
$PAGE
C
C              =========  START OF CODE  ===========
C
C
      IS  = -1
      LUN  = 9
      FNIN = FNAME
C
      MXITO    = 0
C
      RADPLM   = 1.0D0
      SCLPLM   = 1.0D0
      LMXPLM   = 0
      LXUPLM   = -1
      IXUPLM   = 0
C
      MPHASE   =  1
C
      MODNAM   = '      '
C
C
      IF ( LMAX.LT.0 ) GOTO 900
C
      MXLUSE  = LMAX
C
      MXSTOR  = (MXLUSE+1)*(MXLUSE+2)/2
C
      CLOSE ( LUN )
      OPEN ( LUN, FILE=FNIN,STATUS='OLD',IOSTAT=IOCK)
      IF ( IOCK.NE.0 ) GOTO 900
C
      IS  = 1
      NER = 0
C
C
C              INOR  0= NONE, 1=FULL,2=GRAVITY,3=SCHMIDT
C              ITWO = 1 FOR TWO ITEMS / LINE EXCEPT M=0
C              IEXLM = 0, NOT EXPLICIT L,M ON EACH LINE
C              ICS = COS / SIN ALTERNATOR FOR ONELINE (0=COS)
C
      INOR    =  0
      ITWO    =  1
      IEXLM   =  0
      IXL     =  0
      IXM     =  0
C
      ICS     =  0
C
      LMXPLM   =  MXLUSE
      LXUPLM   =  0
      IXUPLM   =  0
C
      LNOW     =  0
      MNOW     =  0
      MXITO    =  0
      MXLS     =  0
C
      IECHO    =  0
C
C              -- PRE CLEAR COEF. ARRAYS --
C
      DO  100 IC = 1, MXSTOR
          CCOEF ( IC )  = 0.0D0
          SCOEF ( IC )  = 0.0D0
100   CONTINUE
C
C
      NREAD  = 0
      NDATA  = 0
C
C
C                       ---------------------------
C
200   CONTINUE
C
      READ ( LUN, 1000, END=800,ERR=700 ) LINE
1000  FORMAT ( A )
C
      IF ( IECHO.NE.0 ) WRITE (*,1201) LINE
1201  FORMAT ( 1X,A79 )
C
      NREAD = NREAD + 1
C
      CALL UPPER ( 4, CMD )
C
      IF ( LINE.EQ.'                             ' ) GOTO 200
      IF ( CMD1.EQ.'*'  ) GOTO 200
      IF ( CMD1.EQ.';'  ) GOTO 200
      IF ( CMD1.EQ.'C'  ) GOTO 200
C
C     ------------------------------------------------
      CALL PRSLIN ( LINE, MXPARM, ITYPV, RVAL, SVAL )
C     ------------------------------------------------
C
      IF ( CMD .EQ. 'NAME' ) GOTO 1100
C 
      IF ( CMD .EQ. 'MAXL' ) GOTO 1200
      IF ( CMD .EQ. 'MAXN' ) GOTO 1200
      IF ( CMD .EQ. 'LMAX' ) GOTO 1200
      IF ( CMD .EQ. 'NMAX' ) GOTO 1200
C
      IF ( CMD .EQ. 'SCAL' ) GOTO 1300
      IF ( CMD .EQ. 'NORM' ) GOTO 1400
      IF ( CMD .EQ. 'TWOL' ) GOTO 1500
      IF ( CMD .EQ. 'ONEL' ) GOTO 1600
      IF ( CMD .EQ. 'LMEX' ) GOTO 1700
      IF ( CMD .EQ. 'END ' ) GOTO 1800
      IF ( CMD .EQ. 'RADI' ) GOTO 1900
C
      IF ( CMD .EQ. 'FILE' ) GOTO 3000
      IF ( CMD .EQ. 'MPHA' ) GOTO 3100
      IF ( CMD .EQ. 'ECHO' ) GOTO 3200
C
C         IF INVALID COMMAND IGNORE
C
      IF ( ITYPV(1).EQ.2 ) GOTO 200
C
C           -----   NOT COMMAND OR COMMENT TRY AND USE   -----
C
      NDATA  = NDATA + 1
C
C
C              IF EXPLICIT L,M ON LINE EXTRACT AND EXAMINE
C
      IF ( IEXLM .NE.0 ) THEN
C
C           PARAMETER IXL   =  L
C                     IXM   =  M
C                       3   =  COS-COEF
C                       4   =  SIN-COEF
C
          IF ( ITYPV(IXL).NE.1 ) GOTO 600
          IF ( ITYPV(IXM).NE.1 ) GOTO 600
C
          IF ( ITYPV(3) .NE. 1 ) GOTO 600
C
          LNOW  = RVAL ( IXL )
          MNOW  = RVAL ( IXM )
C
          IF (LNOW.LT.0 ) GOTO 600
          IF (MNOW.LT.0 ) GOTO 600
          IF (MNOW.GT.LNOW) GOTO 600
C
          IF ( LNOW.GT.MXLUSE ) GOTO 200
C
C         FUDGE FOR M=0,   MAKE CORRESPONDING FIELD VALID AND ZERO
C                          FOR SIN COMPONENT
C
          IF ( MNOW.EQ.0 ) THEN
              ITYPV(4)  = 1
              RVAL(4)   = 0.0D0
          ENDIF
C
          IF ( ITYPV(4) .NE. 1 ) GOTO 600
C
          IF ( INOR.NE.0 ) THEN
              CALL PLMNOR ( INOR, LNOW, MNOW, PLMN, IST )
          ELSE
              PLMN  = 1.0D0
          ENDIF
C
C              M-PHASE ADJUST IF NEEDED
C
          IF ( MPHASE.LT.0 ) THEN
               MEVEN = 2 * (MNOW/2)
               IF ( MEVEN.NE.MNOW )    PLMN  =  - PLMN
          ENDIF
C
C
          SCALE  = SCLPLM / PLMN
C
          ITO  = LNOW * (LNOW+1)/2  + MNOW + 1
C
          CCOEF ( ITO )  =  RVAL ( 3 ) * SCALE
          SCOEF ( ITO )  =  RVAL ( 4 ) * SCALE
C
          IF ( ITO.GT.MXITO ) MXITO = ITO
          IF ( LNOW.GT.MXLS ) MXLS  = LNOW
C
          GOTO 200
C
      ENDIF
C
C              ------------------------
C         STANDARD  NON-EXPLICIT DATA LINE
C         ONE OR TWO ITEMS / LINE
C
C         
C
      IF ( ITYPV (1) .NE. 1 ) GOTO 600
C
C         FUDGE FOR M=0,   MAKE CORRESPONDING FIELD VALID AND ZERO
C                          FOR SIN COMPONENT
C
      IF ( ITWO.EQ.1 .AND. MNOW.EQ.0 ) THEN
              ITYPV(2)  = 1
              RVAL(2)   = 0.0D0
      ENDIF
C
      IF ( ITWO.NE.0 .AND. ITYPV(2).NE.1 ) GOTO 600
C
      SCALE  = SCLPLM
C
      IF ( INOR.NE.0 ) THEN
           CALL PLMNOR ( INOR, LNOW, MNOW, PLMN, IST )
      ELSE
           PLMN  = 1.0D0
      ENDIF
C
C
C              M-PHASE ADJUST IF NEEDED
C
          IF ( MPHASE.LT.0 ) THEN
               MEVEN = 2 * (MNOW/2)
               IF ( MEVEN.NE.MNOW )    PLMN  =  - PLMN
          ENDIF
C
      SCALE  = SCLPLM / PLMN
C
C
C           ---   TWO PER LINE STORAGE IE COS, SIN COEF.  ---
C
      IF ( ITWO.NE.0 ) THEN
C
          ITO  =  LNOW*(LNOW+1)/2 + MNOW + 1
C
          CCOEF ( ITO )  =  RVAL(1) * SCALE
          SCOEF ( ITO )  =  RVAL(2) * SCALE
C
          MXITO  = ITO
          IF ( LNOW.GT.MXLS   ) MXLS = LNOW
C
          MNOW  = MNOW + 1
          IF ( MNOW.GT.LNOW ) THEN
              LNOW = LNOW + 1
              MNOW = 0
          ENDIF
C
C              CHECK IF EXIT ON END OF STORAGE
C
          IF ( LNOW.GT.MXLUSE ) GOTO 500
C
          GOTO 200
C
      ENDIF
C
C         ---  ONE PER LINE /  ALTERNATE COS 1ST THEN SIN  ---
C
      IF ( ITWO.EQ.0 ) THEN
C
C
          ITO  =  LNOW*(LNOW+1)/2 + MNOW + 1
C
          IF ( ICS.EQ.0 ) CCOEF ( ITO )  =  RVAL(1) * SCALE
          IF ( ICS.NE.0 ) SCOEF ( ITO )  =  RVAL(1) * SCALE
C
          MXITO  = ITO
          IF ( LNOW.GT.MXLS   ) MXLS = LNOW
C
          IF ( ICS.EQ.0 .AND. MNOW.NE.0 ) THEN
            ICS = 1
            GOTO 200
          ELSE
            ICS = 0
            MNOW  = MNOW + 1
            IF ( MNOW.GT.LNOW ) THEN
                 LNOW = LNOW + 1
                 MNOW = 0
             ENDIF
          ENDIF
C
C              CHECK IF EXIT ON END OF STORAGE
C
          IF ( LNOW.GT.MXLUSE ) GOTO 500
C
          GOTO 200
C
      ENDIF
C
C              -----------------------------------------
C
C                     VALID   END OF DATA
C
500   CONTINUE
C
      IS  = 0
      GOTO 900
C
C                   --------------------------
C
C                       DATA LINE ERROR
C
600   CONTINUE
C
      IS  = 1
      GOTO 900
C
C                   --------------------------
C
C                         READ ERROR
700   CONTINUE
C
      IS  = 1
      GOTO 900
C
C                   -------------------------
C                       END OF FILE
C
800   CONTINUE
C
      IS  = 0
      GOTO 900
C
C                   -------------------------
C
C                     SOLE RETURN, EXIT CODE
C
900   CONTINUE
C
      CLOSE ( LUN )
C
      LMXPLM   =  MXLUSE
      LXUPLM   =  MXLS
      IXUPLM   =  MXITO
C
      MXIXLD      =  MXITO
C
      A           =  RADPLM
      LMXLD       =  MXLS
      ISTAT       =  IS
C
      RETURN
C
C
$PAGE
C
C
C      =========================================================
C                     COMMAND PROCESSORS 
C                   ----------------------
C
C              NAME
C                  
1100  CONTINUE
C
      IF ( ITYPV(2).NE.2 ) GOTO 200
      MODNAM  = SVAL (2)
C
      GOTO 200
C
C
C              MAXL   MAXIMUM L IN FILE
C
1200  CONTINUE
C
      IF ( ITYPV(2).NE.1 ) GOTO 200
      ML  =  RVAL(2)
C
      IF ( ML.LT.LMAX ) THEN
         MXLUSE  = ML
      ENDIF
C
C
      GOTO 200
C
C
C              SCALE FACTOR
C
1300  CONTINUE
C
      IF ( ITYPV(2) .NE. 1 ) GOTO 200
C
      SCLPLM  = RVAL(2)
C
      GOTO 200
C
C
C              NORMALIZATION TYPE
C
1400  CONTINUE
C
      IF ( ITYPV(2) .NE. 2 ) GOTO 200
C
      CALL UPPER ( 80, SVAL(2) )
      CMD  =  SVAL(2)
C
C
      IF ( CMD .EQ. 'NONE' ) INOR = 0
      IF ( CMD .EQ. 'FULL' ) INOR = 1
      IF ( CMD .EQ. 'GEOP' ) INOR = 2
      IF ( CMD .EQ. 'SCHM' ) INOR = 3
C
      IF ( CMD .EQ. 'GRAV' ) INOR = 2
      IF ( CMD .EQ. 'MAGN' ) INOR = 3
C
C
      GOTO 200
C
C                   TWO LINE
C
1500  CONTINUE
C
      ITWO  =  1
      ICS   =  0
C
      GOTO 200
C
C
C                   ONE LINE
C
1600  CONTINUE
C
      ITWO  =  0
      ICS   =  0
C
      GOTO 200
C
C
C                   LM EXPLICIT VALUES ON LINE
C         
C              COMMAND HAS L-INDX ,  M-INDX AS ARGUMENTS
C
1700  CONTINUE
C
      IF ( ITYPV(2) .NE. 1 ) GOTO 200
      IF ( ITYPV(3) .NE. 1 ) GOTO 200
C
      LX  = RVAL(2)
      MX  = RVAL(3)
C
      IF ( LX.LE.0 .OR. LX.GT.10 ) GOTO 200
      IF ( MX.LE.0 .OR. MX.GT.10 ) GOTO 200
      IF ( MX.EQ.LX ) GOTO 200
C
C              COS, SIN IN FIXED LOCATIONS OF 3, 4
C
      IF ( LX.EQ.3  .OR. LX.EQ.4 ) GOTO 200
      IF ( MX.EQ.3  .OR. MX.EQ.4 ) GOTO 200
C
      IEXLM  = 1
      IXL    = LX
      IXM    = MX
C
C
      GOTO 200
C
C
C                   END COMMAND IN DATA FILE
C
1800  CONTINUE
C
      GOTO 500
C
C                   RADIUS COMMAND
C
1900   CONTINUE
C
C
      IF ( ITYPV(2) .NE. 1 ) GOTO 200
C
      RADPLM  = RVAL(2)
C
      GOTO 200
C
C
C              ------  NEW FILE FOR INPUT   ------
C
3000   CONTINUE
C
      IF ( ITYPV(2).NE.2 ) GOTO 200
      FNIN = SVAL(2)
C
      IS   =  -2
C
      NSKP  =  0
C
      IF ( ITYPV(3) .EQ. 1 ) NSKP = RVAL(3)
C
      CLOSE ( LUN )
      OPEN ( LUN, FILE=FNIN,STATUS='OLD',IOSTAT=IOCK)
      IF ( IOCK.NE.0 ) GOTO 900
C
      IF ( NSKP.GT.0 ) THEN
         DO   3010  ISKP = 1, NSKP
            READ ( LUN,1003,END=900,ERR=900 ) CDUM
1003        FORMAT ( A1)
3010     CONTINUE
C
      ENDIF
C
      GOTO  200
C
C              M- PHASE SET / RESET
C
3100   CONTINUE
C
      IF ( ITYPV(2).NE.1 ) GOTO 200
      MPHASE = 1
      IF ( RVAL(2).LT.0.0D0 ) MPHASE = -1
C
      GOTO 200
C
C              ECHO LIST ON
C
3200  CONTINUE
C
      IECHO = 1
      IF ( ITYPV(2).NE.2 ) GOTO 200
      CMD  = SVAL(2)
      CALL UPPER ( 4, CMD )
      IF ( CMD.EQ.'OFF ' ) IECHO = 0
C
      GOTO 200
C
C                ========  PROGRAM  END  =========
C
C
      END
