       FUNCTION Q FIT( N,XA,YA,X) 
C 
C---INTERPOLATES FOR A "Y" WHEN GIVEN A "X" USING A DOUBLE QUADRATIC FIT
C 
C  THE FIT IS THRU 2 SETS OF 3 POINTS ENCLOSING THE X.
C  THE FUNCTIONAL VALUE IS THE AVERAGE OF THESE QUADRATICS AT X.
C  IF ONLY 1 SET OF POINTS ENCLOSES X, "Y" IS BASED ON A SINGLE QUADRATI
C  IF X IS NOT ENCLOSED BY ANY POINTS, "Y" IS AN EXTRAPOLATION
C                                             BASED ON A SINGLE QUADRATI
C 
C           INPUT: N=NUMBER OF POINTS IN THE XA & YA ARRAYS 
C                  XA=ARRAY OF X POINTS 
C                  YA=ARRAY OF Y POINTS 
C                  X=THE X VALUE MISSING A Y
C 
C           NOTES: -XA SHOULD BE IN ASCENDING ORDER.
C                  -XA AND YA SHOULD BE THE SAME LENGTH.
C                  -BOTH XA & YA SHOULD BE DIMENSIONED A MINIMUM
C                    OF 3 IN THE CALLING ROUTINE AND HAVE AT
C                    LEAST 3 DATA POINTS. 
C 
C   LAST MODIFIED: KAH/JRC ADJUST X TO 0 IN FIT 6-29-84 
C                  AGC/DSC DOCUMENT & FIX ARRAY OVERWRITE IN JLOOP 22/07
C                  JRC     7-89 MOVED TO PC
C 
C-----------------------------------------------------------------------
C
$INCLUDE:'WRDSIZ.INC'
C
C 
      DIMENSION XA(1),YA(1), ABC(3,2) 
      DIMENSION XUSE(4),YUSE(4) 
C 
      DATA NB,NI  / -1, -1 / 
C 
C---(* INITIALIZATION *)
C 
    5 CONTINUE
      NB=1
      NI=N
C 
C---(* GOTO REFERENCE FOR REPEAT ENTRIES --QFITC *) 
C 
   10 CONTINUE
      I=NB-2
      IF(I.LT.1) I=1
C 
C---(* I LOOP TO FIND XA(I) > X *)
C 
   20 CONTINUE
C
      IF(XA(I).GT.X) GOTO 30
      I=I+1 
      IF(I.LE.NI) GOTO 20 
C
C---(* I=NI IF X > ALL OF XA --USE FOREWARD EXTRAPOLATION *)
C
      I=NI
C
   30 CONTINUE
C 
C      FIND TWO SETS OF QUADRITIC EQUATION COEFICIENTS (A,B,C)
C      Y =A*X**2 + B*X + C
C      FROM TWO DATA SETS OF 3 POINTS CONTAINING X
C 
C---(* IX IS INDEX OF THE XA & YA VALUES IN "-BIAS" AND 
C                         HELPS DETERMINE WHERE "-USE" START *) 
C
      IX=I-2
      IF(IX.LT.1) IX=1
C 
C 
C     ADDED XBIAS AND YBIAS . 6/19/84. KAH
C 
C 
C---(* "-BIAS" ARE ADJUSTMENTS TO SLIDE GRAPH TO THE ORIGIN *)
C 
      XBIAS=XA(IX)
      YBIAS=YA(IX)
C 
C---(* JMAX CORRECTION PREVENTS INDEFINITIVE ERROR IN J LOOP *) 
C                                      A.CHU   22/07/85 
C
      IF (IX+3.GT.NI) JMAX=3
      IF (IX+3.LE.NI) JMAX=4
C 
C---(* J LOOP TO GENERATE "-USE" ARRAYS SENT TO "QCOEF" *)
C 
      DO 40 J=1,JMAX
         IFRM=J-1+IX
         XUSE(J)=XA(IFRM)-XBIAS 
         YUSE(J)=YA(IFRM)-YBIAS 
   40 CONTINUE
C 
C---(* QCOEF FINDS THE COEFICIENTS FOR THE QUADRATIC EQUATION 
C                         CONTAINING THE POINTS IN "-USE" ARRAYS *) 
C---(* MOVLEV PUTS THE 1ST SET OF COEFS IN THE 2ND SET OF COEFS 
C                       IN CASE THE 2ND CALL TO QCOEF IS SKIPPED *) 
C 
C
      CALL Q COEF (XUSE(1),YUSE(1),ABC(1,1))
      CALL MOVRA (3,ABC(1,1), ABC(1,2))
C 
C---(* SKIP 2ND QCOEF FOR BACKWARDS EXTRAPOLATION OR
C                                           X TOO NEAR BEG OF XA *) 
C
      IF((I-2).NE.IX) GOTO 50 
C 
C---(* SKIP 2ND QCOEF FOR FOREWARDS EXTRAPOLATION OR
C                                           X TOO NEAR END OF XA *) 
C
      IX=IX+1 
      IF((IX+2).GT.NI) GOTO 50
C 
C---(* SECOND CALL TO QCOEF --CALLED ONLY IF THERE ARE 2 GOOD PTS 
C                                          ON BOTH SIDES OF THE X *)
C
      CALL Q COEF (XUSE(2),YUSE(2),ABC(1,2))
C 
C---(* AVERAGE THE COEFICIENT VALUES *) 
C 
   50 CONTINUE
C
      A  =   0.5*(ABC(1,1) + ABC (1,2)) 
      B  =   0.5*(ABC(2,1) + ABC (2,2)) 
      C  =   0.5*(ABC(3,1) + ABC (3,2)) + YBIAS 
C 
C---(* XEV IS X VALUE ADJUSTED TO "-USE" ARRAYS *)
C
      XEV = X - XBIAS 
C 
C---(* CALCULATE THE Y FOR X -- Y = AX}+BX+C = X(AX+B)+C -- *) 
C 
      VALUE  =   C + XEV * ( B+ XEV*A )
      QFIT   =   VALUE
C 
C---(* SAVE NEW BEG POINT OF I LOOP FOR RE-ENTRY *) 
C 
      NB=IX-3 
      RETURN
C 
C==================  COMMENT OUT ALTERNATE ENTRY  ===============
C      ENTRY Q FIT C 
C 
C---(* IF THIS DOES NOT APPEAR TO BE A RE-ENTRY THEN GOTO 5 
C                                       ELSE SKIP INITIALIZATION *) 
C      IF(N.NE.NI) GOTO 5
C      IF(NB.LE.0) GOTO 5
C      GOTO 10 
C 
C  ==============================================================
C---(* THIS IS THE END----FINALLY!!!! *)
C 
      END 
$PAGE
$SUBTITLE:'QUADRATIC COEFS'
      SUBROUTINE Q COEF( XA, YA, ABC) 
C
C         FIND QUADRIC COEF TO 3 POINTS
C
C      INPUT
C            XA(1...3)
C            YA(1..3)
C      OUTPUT
C              ABC(1..3) = A,B,C  : Y= AX**2 + BX + C
C
C     MODIFICATIONS:  JRC 7-89 TO PC
C
$INCLUDE:'WRDSIZ.INC'
C
C 
      DIMENSION XA(3), YA(3)
      DIMENSION ABC(3), COEF(3,3), COEFI(3,3) 
C 
C     FIND QUADRATIC COEF FOR 3 POINTS
C 
       X1=XA(1) 
       X2=XA(2) 
       X3=XA(3)
C 
      COEF(1,3) = 1.0
      COEF(2,3) = 1.0
      COEF(3,3) = 1.0 
C
      COEF(1,2)=X1  
      COEF(2,2)=X2  
      COEF(3,2)=X3
C
      COEF(1,1)=X1**2 
      COEF(2,1)=X2**2 
      COEF(3,1)=X3**2 
C 
      SUMSQ= COEF(1,1) + COEF(2,1) + COEF(3,1)
      SUMABS= ABS(X1) + ABS(X2) + ABS(X3) 
      TESTDET= 1.0E-13* SUMSQ * SUMABS
C 
      IFLAG= MXIV33 (COEF,COEFI,DET)
      IF(IFLAG.NE.0) GOTO 100 
      IF(ABS(DET).LT.TESTDET) GOTO 100
C 
      CALL MXV33 (COEFI,YA,ABC)
C 
      RETURN
C 
100   CONTINUE
C
      Y1= YA(1)  
      Y2= YA(2)
C
      DX= X2-X1  
      DY= Y2-Y1
C
      A   =   0.0 
      B   =   0.
      C   =   Y1
C
      TESTDX= 1.0E-13*(ABS(X1) + ABS(X2)) 
      IF(ABS(DX).LT.TESTDX) GOTO 150
C
      B= DY/DX
      C= Y1 - B*X1
C
150   CONTINUE
C 
      ABC(1)=A 
      ABC(2)=B 
      ABC(3)=C
C
      RETURN
      END 
