      SUBROUTINE QFITV ( LEN, IXX, N, ARRAY, X, VOUT )
C
C         QFIT OF VECTOR 
C              [ OLD QFIT ALGORITHM ..   
C                FIND X IN INTERVAL AND TAKE 2 PTS EACH SIDE
C                FIND TWO QUADRATICS, 1ST 3 PTS, LAST 3 PTS AND
C                AVERAGE FIT OF TWO SETS]
C
C    INPUT:
C              LEN       I*4       LENGTH OF EACH VECTOR
C                                  LEN .GE.4 AND .LE. 60
C              IXX       I*4       INDEX OF X VALUE (USUALLY 1 )
C              N         I*4       NUMBER OF VECTORS IN MATRIX
C              ARRAY ( LEN, N )  R*8
C                                  N VECTORS OF LENGTH LEN.  THIS IS
C                                  DATA TABLE TO USE IN FITS.  INDEX IXX
C                                  IS THE INDEPENDENT VARIABLE
C              X         R*8       VALUE AT WHICH TO FIT ALL OTHER PARAMETERS
C    OUTPUT:
C              VOUT(LEN) R*8       VOUT(IXX) = X, OTHERS ARE INTERPOLATION
C                                  OF PARAMETERS AT X.
C
C         ASSUMES THAT X VALUES ARE MONTONICLY INCREASING
C         
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION     ARRAY ( LEN, N ), VOUT (LEN)
C
      DIMENSION     VOUT1(60), VOUT2(60)
C
C         -----------------------------------------------------
C
C
C         SANITY CHECKS -- DO NOTHING IF ERROR
C
      IF ( LEN.LE.2 ) GOTO 900
      IF ( LEN.GT.60 ) GOTO 900
      IF ( IXX.LT.LEN) GOTO 900
      IF ( N.LT.3 ) GOTO 900
C
C
C         SEARCH TILL FIND AREA STRADDLES X, OR OFF ONE END OF TABLE
C
      XI   = X
C
C
      XL  = ARRAY ( IXX,1 )
      XE  = ARRAY ( IXX,N )
C
      IF ( XI.LE.XL ) THEN
          IBEG  = 1
          IBEGP = 1
          GOTO 200
      ENDIF
C
      IF ( XI.GE.XE ) THEN
           IBEG  = N - 2
           IBEGP = IBEG
           GOTO 200
      ENDIF
C
      DO  100  IP = 2, N
         IXB= IP
         XN = ARRAY ( IXX, IXB )
         IF ( XI.LE.XN .AND. XI.GT.XL ) GOTO 120
         XL  = XN
100   CONTINUE
C    
120   CONTINUE
C
      IBEG = IXB - 2
      IF ( IBEG.LT.1 ) IBEG = 1
      IEND = IBEG + 3
      IF ( IEND.GT.N ) THEN
         IEND = N
         IBEG = IEND - 2
      ENDIF
      IBEGP = IBEG + 1
      IENDP = IEND + 1
      IF ( IENDP.GT.N ) IBEGP = IBEG
C
200   CONTINUE
C
C         DO TWO LAGRANGIAN 3 POINT INTERPOLATIONS
C         ( THE POLYNOMIAL IS "THE" POLYNOMIAL SO ANYWAY GET IS SAME)
C            
      CALL LAG3V ( LEN, IXX, ARRAY(1,IBEG),  X, VOUT1 )
      IF ( IBEGP.NE.IBEG ) THEN
        CALL LAG3V ( LEN, IXX, ARRAY(1,IBEGP), X, VOUT2 )
      ENDIF
C
C              AVERAGE THE TWO
C
      IF ( IBEG.NE.IBEGP ) THEN
         DO 400 I = 1, LEN
            VOUT(I) = 0.5D0 * ( VOUT1(I) + VOUT2(I) )
400      CONTINUE
      ELSE
         DO 410 I = 1, LEN
            VOUT(I) =  VOUT1(I) 
410      CONTINUE
      ENDIF
C
C
900   CONTINUE
C
      END
$page
      SUBROUTINE QFITXV ( LEN, N, XA, ARRAY, X, VOUT )
C
C         QFIT OF VECTOR 
C              [ OLD QFIT ALGORITHM ..   
C                FIND X IN INTERVAL AND TAKE 2 PTS EACH SIDE
C                FIND TWO QUADRATICS, 1ST 3 PTS, LAST 3 PTS AND
C                AVERAGE FIT OF TWO SETS]
C
C    INPUT:
C              LEN       I*4       LENGTH OF EACH VECTOR
C                                  LEN .GE.4 AND .LE. 60
C              N         I*4       NUMBER OF VECTORS IN MATRIX
C              XA(N)     R*8       ARRAY OF INDEPENDENT VARIABLE
C              ARRAY ( LEN, N )  R*8
C                                  N VECTORS OF LENGTH LEN.  THIS IS
C                                  DATA TABLE TO USE IN FITS. 
C              X         R*8       VALUE AT WHICH TO FIT ALL OTHER PARAMETERS
C    OUTPUT:
C              VOUT(LEN) R*8       INTERPOLATION  OF PARAMETERS AT X.
C
C         ASSUMES THAT X VALUES ARE MONTONICLY INCREASING
C         
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION     XA ( N ), ARRAY ( LEN, N ), VOUT (LEN)
C
      DIMENSION     VOUT1(60), VOUT2(60)
C
C         -----------------------------------------------------
C
C
C         SANITY CHECKS -- DO NOTHING IF ERROR
C
      IF ( LEN.LE.2 ) GOTO 900
      IF ( LEN.GT.60 ) GOTO 900
      IF ( N.LT.3 ) GOTO 900
C
C
C         SEARCH TILL FIND AREA STRADDLES X, OR OFF ONE END OF TABLE
C
      XI   = X
C
C
      XL  = XA ( 1 )
      XE  = XA ( N )
C
      IF ( XI.LE.XL ) THEN
          IBEG  = 1
          IBEGP = 1
          GOTO 200
      ENDIF
C
      IF ( XI.GE.XE ) THEN
           IBEG  = N - 2
           IBEGP = IBEG
           GOTO 200
      ENDIF
C
      DO  100  IP = 2, N
         IXB= IP
         XN = XA ( IXB )
         IF ( XI.LE.XN .AND. XI.GT.XL ) GOTO 120
         XL  = XN
100   CONTINUE
C    
120   CONTINUE
C
      IBEG = IXB - 2
      IF ( IBEG.LT.1 ) IBEG = 1
      IEND = IBEG + 3
      IF ( IEND.GT.N ) THEN
         IEND = N
         IBEG = IEND - 2
      ENDIF
      IBEGP = IBEG + 1
      IENDP = IEND + 1
      IF ( IENDP.GT.N ) IBEGP = IBEG
C
200   CONTINUE
C
C         DO TWO LAGRANGIAN 3 POINT INTERPOLATIONS
C         ( THE POLYNOMIAL IS "THE" POLYNOMIAL SO ANYWAY GET IS SAME)
C            
      CALL LAG3XV ( LEN, XA(IBEG), ARRAY(1,IBEG),  X, VOUT1 )
      IF ( IBEGP.NE.IBEG ) THEN
        CALL LAG3XV ( LEN, XA(IBEGP), ARRAY(1,IBEGP), X, VOUT2 )
      ENDIF
C
C              AVERAGE THE TWO
C
      IF ( IBEG.NE.IBEGP ) THEN
         DO 400 I = 1, LEN
            VOUT(I) = 0.5D0 * ( VOUT1(I) + VOUT2(I) )
400      CONTINUE
      ELSE
         DO 410 I = 1, LEN
            VOUT(I) =  VOUT1(I) 
410      CONTINUE
      ENDIF
C
C
900   CONTINUE
C
      END
