$TITLE:'ROBUST ESTIMATE1'
      SUBROUTINE RBUSTE ( N, A, EVAR, RBOUT, IWORK )
C
C         ROBUST ESTIMATE OF VALUES
C
C         USES ORDINAL VALUES
C
C      INPUT:
C              N         I*4       NUMBER OF VALUES IN INPUT
C              A         R*8       INPUT ARRAY N LONG
C              EVAR      R*8       EXPECTED VARIATION
C                                  (VALUES WITHIN THIS TOL. OR OTHERS
C                                  EFFECTIVELY THE SAME FOR ORDERING )
C      OUTPUT:
C              RBOUT     R*8       OUTPUT ARRAY
C                   (1)            AVERAGE ALL VALUES
C                   (2)            STANDARD DEVIATION ALL VALUES
C                   (3)            MIN ALL VALUES
C                   (4)            MAX ALL VALUES
C                   (5)            MEDIAN VALUE
C                   (6)            ROBUST ESTIMATE
C                   (7)            STANDARD DEVIATION ROBUST VAL
C                   (8)            MAX VALUES IN ROBUST ESTIMATE
C                   (9)            MIN     "   "    "     "      
C                  (10)            QUALITY--NUMBER POINTS USE ROB.EST
C                                       OR
C                                  -2  DATA ERROR
C                                  -1  ALL BUT ROBUST EST FILLED IN
C      USE
C             IWORK (N)   I*4      WORK ARRAY N LONG ( USED FOR POINTERS)
C                                  LEFT AS INDEX'S , ORDERING OF ARRAY
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION   A(*),IWORK(*), RBOUT(7)
C
C   ------------------------------------------------------
C
      FUZZ  = ABS ( EVAR )
C
      NPARM = 10
      DO 10 I = 1, 10
         RBOUT(1) = 0.0D0
10    CONTINUE
C
      NI  = N
C
      RBOUT (10)  = -2.0
C
      IF ( N.LE.0 ) GOTO 900
C
C       SORT THE ARRAY
C
      CALL PSRTR8 ( N, A, IWORK )
C
C         GET MEDIAN VALUE
C
      MEDN   = NI/2 + 1
      IXMED  = IWORK ( MEDN )
      XMED   = A ( IXMED )
C
C         SCAN ARRAY FOR AVG, SD, MIN, MAX ETC
C
      XMAX  = -1.0D25
      XMIN  = - XMAX
C
      ASUM  = 0.0D0
      AASUM = 0.0D0
C
      DO 100 I = 1, NI
         X     = A(I)
         ASUM  = X   + ASUM
         AASUM = X*X + AASUM
         IF ( X .GT. XMAX ) XMAX = X
         IF ( X .LT. XMIN ) XMIN = X
100   CONTINUE
C
      FN     = NI
      FNM1   = NI - 1
      IF ( NI.LE.1 )  FNM1 = FN
C
      AVG    = ASUM / FN
      RMS    = SQRT ( AASUM/FN )
      VAR    =  (AASUM - ASUM*ASUM/FN )/FNM1
      SIG    = SQRT ( DABS(VAR) )
C
      DIF    =  XMAX - XMIN
C
C        FILL IN THE VALUES NOW HAVE
C
      RBOUT(1)  =  AVG
      RBOUT(2)  =  SIG 
      RBOUT(3)  =  XMIN
      RBOUT(4)  =  XMAX
      RBOUT(5)  =  XMED
      RBOUT(10)  = -1.0
C
C              NOW MOVE OUT FROM MEDIAN AND GET AVERAGE
C              STOP IN EITHER DIRECTION WHEN
C                   DIF FROM CURRENT ESTIMATE .GT. LIMIT
C                   OUT OF DATA
C  
C               LIMIT  = 5 * EVAR  OR 5 * LAST DIF
C
C       MOVE TO NEXT POINT WITH MIN DIF FROM MEDIAN
C
      IDOWN = 1
      IUP   = 1
      IXD   = MEDN - 1
      IXU   = MEDN + 1
C
      DFTST   = 3.0 * FUZZ
      DLAST   = XMED
      ULAST   = XMED
C
      FNUSE   = 1
      XSUM    = XMED
      XXSUM   = XMED*XMED
C
      XMAX  = XMED    
      XMIN  = XMED
C
200   CONTINUE
C
      IF ( IXD .LE. 0 ) IDOWN = 0
      IF ( IXU .GT. NI) IUP   = 0
C
C          LOOP EXIT TEST
C
      IF ( (IDOWN+IUP) .EQ. 0 ) GOTO 500
C
C          GET VALUES, FIND WHICH ITEM TO USE
C
      IF ( IUP.NE. 0  ) THEN
           IXUP  = IWORK ( IXU )
           XUP   = A(IXUP)
           DUP   = ABS ( XUP - XMED )
      ENDIF
C
      IF ( IDOWN.NE.0 ) THEN
           IXDW  =  IWORK ( IXD )
           XDW   =  A(IXDW)
           DDW   =  ABS ( XDW - XMED )
      ENDIF
C
C        IF -- DETERMINE ONE TO USE -- ONE ON OR MIN DIF FROM
C                                      MEDIAN
C
      IF ( IUP.NE.0  .AND. IDOWN.NE.0 ) THEN
           IUSE = +1
           IF ( DDW .LT. DUP ) IUSE = -1
      ELSE
           IUSE = +1
           IF ( IUP.EQ.0 ) IUSE = -1
      ENDIF
C
      IF ( IUSE.EQ.1 ) THEN
C
          D   = ABS ( XUP - ULAST )
          IF ( D .GT. DFTST ) THEN
             IUP  = 0
             GOTO 200
          ENDIF
C
          XSUM  = XSUM + XUP
          XXSUM = XXSUM + XUP*XUP
          FNUSE = FNUSE + 1.0D0
C
          IF(  XUP .GT. XMAX ) XMAX = XUP
          IF(  XUP .LT. XMIN ) XMIN = XUP
C
C
          ULAST = XUP
          IXU   = IXU + 1
C
      ENDIF
C
      IF ( IUSE.EQ.-1 ) THEN
C
          D   = ABS ( XDW - DLAST )
          IF ( D .GT. DFTST ) THEN
             IDOWN  = 0
             GOTO 200
          ENDIF
C
          XSUM  = XSUM + XDW
          XXSUM = XXSUM + XDW*XDW
          FNUSE = FNUSE + 1.0D0
C
          IF(  XDW .GT. XMAX ) XMAX = XDW
          IF(  XDW .LT. XMIN ) XMIN = XDW
C
          DLAST  = XDW
          IXD    = IXD - 1
C
      ENDIF
C
      DFTST  = 5.0D0 * D
      IF ( DFTST .LT. FUZZ)  DFTST = FUZZ
C
      GOTO 200
C
C             NOW COMPUTE ROBUST ESTIMATE
C
500   CONTINUE
C
      IF ( FNUSE .GT. 2.0D0 ) THEN
C
          AVGRB  =  XSUM / FNUSE
          VARRB  =  (XXSUM - XSUM*XSUM/FNUSE) / ( FNUSE-1.0D0 )
          SIGRB  =  SQRT ( VARRB )          
C
          RBOUT(6)  = AVGRB
          RBOUT(7)  = SIGRB
          RBOUT(8)  = XMIN
          RBOUT(9)  = XMAX
C
          RBOUT(10) = FNUSE
C
      ENDIF
C
900   CONTINUE
C
      RETURN
      END
