$TITLE:'SPHERICAL HARMONIC ROUTINES'
$SUBTITLE:'ASSOC. LEGENDRE --N.R. ALGORITHM'
      SUBROUTINE PLM  ( L, M, X, PLMVAL, ISTAT )
C
C          COMPUTE THE ASSOCIATED LEGENDRE POLYNOMINAL
C
C         Based on Numerical Receipes
C
C    MODHIST:
C              1-31-94   J.R. Clynch Coded
C
C    INPUT:
C
C              L         I*4       L OF Plm (X),  NORMAL LEGENDRE INDEX
C                                  L in [0,..N ] 
C
C              M         I*4       M "   ",   M in [0..L]
C
C              X         R*8       ARGUMENT,  X in [-1, 1]
C
C    OUTPUT:
C
C              PLM       R*8       VALUE OR 0.0 IF ERROR
C              ISTAT     I*4       0  OKAY
C                                  1  ERROR, INVALID INPUT
C
C              ----------------------------------------------
C
C         ALGORITHM   __Horizontal Stepping__
C
C         |                              .                       
C         |                            . ----- M = L line        
C         |                          .                           
C         |                        .          VALID AREA         
C         |                      .                               
C         |                    .                                 
C    M    |                  .                                   
C         |  Begin Here    .                                     
C         |  at M,M ---> x y XXXXXXXXXX*<---  Required L,M       
C         |  (EQ 3)    .   | |                                   
C         |          .     | |__General Horizontal Stepping   (EQ 1)
C         |        .       |___ Initial extra value (EQ.2)       
C         |      .                                               
C         |    .                                                 
C         |  .                                                   
C         |.                                                     
C         ----------------------------------------------------
C    
C                             L 
C
C
C
C
C
C         L  STEP General
C    
C   1.      ( L-M )*P[L,M]    =  X*( 2*L-1 ) * P[L-1,M]  
C                                - ( L+M-1 ) * P[L-2,M]
C
C         L END POINT EQ  ( M=L)
C
C                        TAKEN OUT -1**m   2/21/94
C                        -----------
C   2.      P[L,L]     =  ( 2L-1 )!! * ( 1-X*X )**(L/2)
C
C         L 1ST Step Over  ( EQ 1 SPECIAL CASE )
C
C   3.      P[L+1,L]  =  X*( 2*L+1 ) * P[L,L]
C
C         WILL BEGIN ON DIAGNAL AT  L = M,
C         FIND ITEM BELOW BY EQ 3
C         CONTINUE OVER WITH EQ 1 ( NEEDS TWO PREVIOUS )
C
C         IE START AT CORRECT M, BUT L LESS (= M ) AND GO UP IN L
C         TILL GET TO DESIRED L,M.
C
C                ---------------------------------------
C
$INCLUDE:'WRDSIZ.INC'
C
C    -----------------------------------------------------------------
C
C        DEFAULT RETURNS AND VALID ARGUMENT CHECKS
C
      IS    = 1
      VAL   = 0.0D0
C
      IF ( L.LT.0 ) GOTO 900
      IF ( M.LT.0 ) GOTO 900
      IF ( M.GT.L ) GOTO 900
      IF ( DABS(X) .GT. 1.0D0 ) GOTO 900
C
C                  ---------------------
C
C         INITIAL VALUE,  COMPUTE P[M,M]
C
      SQ1MXS   =   SQRT ( 1.0D0  -  X * X )
C
      PMM      =   1.0D0
      IF ( M.EQ.0 ) GOTO 120
C
      FACT     =   1.0D0
C
      DO  100 I = 1, M
         PMM  =   PMM * FACT * SQ1MXS
         FACT =   FACT + 2.0D0
100   CONTINUE
120   CONTINUE
C
      IF ( L.EQ.M ) THEN
          VAL  = PMM
          GOTO  600
      ENDIF
C
C              L .GT.  M, MUST AT LEAST INCREASE L BY 1
C              FIRST STEP EQUATION  ( P[M+1,M] )
C
      PLP1M  =  ( 2 * M + 1 ) * X * PMM
C
C         CK IF GOT VALUE, ELSE DO GENERALIZED STEP UP IN L
C         USING TWO PREVIOUS VALUES
C
      IF ( (L-1).EQ.M ) THEN
           VAL  =  PLP1M
           GOTO 600
      ENDIF
C
C
      PLM2  = PMM
      PLM1  = PLP1M
C
      LNXT  =  M+2
      DO  200 LL = LNXT, L
C
         DIV   =  LL - M
         FNUM  =  ( 2*LL-1 ) * X * PLM1 - (LL+M-1) * PLM2
C
         VAL   =  FNUM / DIV
C
         PLM2  =  PLM1
         PLM1  =  VAL
C
200   CONTINUE
C
C              VALID END
C
600   CONTINUE
C
      IS  = 0
C
900   CONTINUE
C
      ISTAT   = IS
      PLMVAL  = VAL
C
      RETURN
      END
$PAGE
$SUBTITLE:'ASSOC. LEGENDRE -- ARRAY'
      SUBROUTINE PLMARY ( LMAX, X, PLMA, ISTAT )
C
C          COMPUTE THE ASSOCIATED LEGENDRE POLYNOMINAL
C           --- ENTIRE ARRAY [0,0]  TO  [L,L]
C
C         Based on Numerical Receipes
C
C    MODHIST:
C              2-5-94   J.R. Clynch Coded
C              2-21-94  CHANGED NORM. TOOK OUT -1**M
C
C    INPUT:
C
C              LMAX      I*4       LMAX Plm (X), IE TP Plm[L,L]
C
C              X         R*8       ARGUMENT,  X in [-1, 1]
C
C    OUTPUT:
C
C              PLMA      R*8       VALUE ARRAY (L+1)*(L+2)/2 LONG
C              ISTAT     I*4       0  OKAY
C                                  1  ERROR, INVALID INPUT
C
C         STORAGE
C
C                   [0,0],[1,0],[1,1],[2,0],[2,1]....
C
C                   EACH  L  SEGEMENT:
C
C                          L+1 ELEMENTS LONG
C
C                          BEGINS AT  1 + L*(L+1)/2
C                                                                     
C
C
C       |                                                             
C       |     CELL INDEX                                              
C       |                                                             
C       |                      15                                          
C       |                               STORAGE ORDER: UP L'S          
C       |                 10   14                                          
C  M    |                               Computation,  Horizontal      
C       |             6    9   13       Out fixed M's                      
C       |                                                             
C       |        3    5    8   12                                          
C       |                                                             
C       |   1    2    4    7   11                                           
C       |____________________________________________________         
C           1    2    3    4    5    6    7                           
C                                                                     
C                             L
C
C
C
C              ----------------------------------------------
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION   PLMA ( * )
C
C    -----------------------------------------------------------------
C
C        DEFAULT RETURNS AND VALID ARGUMENT CHECKS
C
      IS    = 1
C
      IF ( LMAX.LT.0 ) GOTO 900
      IF ( DABS(X) .GT. 1.0D0 ) GOTO 900
C
C
      SQ1MXS   =   SQRT ( 1.0D0  -  X * X )
C    
C
C                  ---------------------
C
C         DO ONE ROW BY HORIZONTAL STEPPING AT A TIME
C
      NDO  =  LMAX + 1
      L    =  LMAX
C
      DO   600  MP1  = 1, NDO
C
C              CURRENT M, NUMBER OF L'S TO DO
C
           M     = MP1 - 1
           NL    = 1 + L - M
C
C              BEGINNING INDEX,  AND D-INDEX 
C
           INDX  = (M+1)*(M+2)/2
           IXDN  =  M + 1
C
           LL  =  M
C
C          -- INITIAL VALUE,  COMPUTE P[M,M]  FROM P[M-1,M-1] --
C
           IF ( M.EQ.0 ) THEN
                PMMN  = 1.0D0
                PMM   = 1.0D0
           ELSE
               PMM      =      (2*M-1) * SQ1MXS * PMMN
               PMMN     =       PMM
           ENDIF
C
           PLMA ( INDX )  = PMM
           INDX           = INDX + IXDN
           IXDN           = IXDN + 1
           LL             = LL   + 1
C
           IF ( NL.EQ.1 ) GOTO 600
C
C           --   FIRST STEP EQUATION  ( P[M+1,M] ) --
C
           PLP1M  =  ( 2 * M + 1 ) * X * PMM
C
           PLMA ( INDX )  =  PLP1M
           INDX           =  INDX + IXDN
           IXDN           =  IXDN + 1
           LL             =  LL   + 1
C
           IF ( NL.EQ.2 ) GOTO 600
C
C
           PLM2  = PMM
           PLM1  = PLP1M
C
           LNXT  =  LL 
C
           DO  200 LL = LNXT, L
C
              DIV   =  LL - M
              FNUM  =  ( 2*LL-1 ) * X * PLM1 - (LL+M-1) * PLM2
C
              VAL   =  FNUM / DIV
C
              PLMA ( INDX )  =  VAL
              INDX           =  INDX + IXDN
              IXDN           =  IXDN + 1
C
              PLM2  =  PLM1
              PLM1  =  VAL
C
200        CONTINUE
C
C
600   CONTINUE
C
      IS  = 0
C
900   CONTINUE
C
      ISTAT  = IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM NORMALIZATION CONST.'
      SUBROUTINE  PLMNOR ( ITYPE, L,M, PLMN, ISTAT )
C
C         NORMALIZATION OF Plm
C
C         ITYPE = 0      NONE  ( Returns 1 )
C         ITYPE = 1      FULL
C         ITYPE = 2      Gravity ( Geophysics )
C         ITYPE = 3      Schmidt ( Magnetic Field type )
C         ITYPE = OTHER  1.0 AND ERROR SET
C
C                  ==================================
C         =======  DIVIDE Plm BY THESE NORMALIZATIONS  =========
C                  ==================================
C
C
C    MODHIST:
C              2-1-94    CODED  JRC
C              2-11-94   RENAMED SO CAN HAVE UPPER LEVEL ROUTINE
C
C
C         VALUE**2  =  {2/(2*L+1)} * { (L+M)! } / { (L-M)! }
C
C
C    INPUT:
C              ITYPE     I*4       TYPE OF NORMALIZATION NEEDED
C
C                                 ITYPE = 0      NONE  ( Returns 1 )
C                                 ITYPE = 1      FULL
C                                 ITYPE = 2      Gravity ( Geophysics )
C                                 ITYPE = 3      Schmidt ( Mag. Field )
C                                 ITYPE = OTHER  1.0 AND ERROR SET
C
C              L         I*4       MAJOR INDEX
C              M         I*4       MINOR ( SECTORAL ) INDEX
C
C    OUTPUT:
C              PLMN      R*8       NORMALIZATION CONSTANT
C                                  SQRT OF INTEGERAL
C              ISTAT     I*4       0, OKAY, 1 INVALID INPUT
C
C
$INCLUDE:'WRDSIZ.INC'
C
C              -------------------------------------
C
      VAL  =   1.0D0
      IS   =  -1
      IF ( ITYPE.LT.0 .OR. ITYPE.GT.3 ) GOTO 900
C
      IF ( ITYPE.EQ.0 ) THEN
         IS   = 0
         GOTO  900
      ENDIF
C
      IF ( ITYPE.EQ.1 ) CALL PLMNFL (  L,M, VAL , IS )
      IF ( ITYPE.EQ.2 ) CALL PLMNGP (  L,M, VAL , IS )
      IF ( ITYPE.EQ.3 ) CALL PLMNSC (  L,M, VAL , IS )
C
900   CONTINUE
C
      PLMN   =   VAL
      ISTAT  =   IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM FULL NORMALIZATION CONST.'
      SUBROUTINE  PLMNFL ( L,M, PLMN, ISTAT )
C
C         FULL NORMALIZATION OF Plm
C              IE =  INTEGRAL ( Plm*Plm ) FROM -1,1
C
C                  ==================================
C         =======  DIVIDE Plm BY THESE NORMALIZATIONS  =========
C                  ==================================
C
C
C    MODHIST:
C              2-1-94    CODED  JRC
C              2-11-94   RENAMED SO CAN HAVE UPPER LEVEL ROUTINE
C
C
C         VALUE**2  =  {2/(2*L+1)} * { (L+M)! } / { (L-M)! }
C
C
C    INPUT:
C              L         I*4       MAJOR INDEX
C              M         I*4       MINOR ( SECTORAL ) INDEX
C
C    OUTPUT:
C              PLMN      R*8       NORMALIZATION CONSTANT
C                                  SQRT OF INTEGERAL
C              ISTAT     I*4       0, OKAY, 1 INVALID INPUT
C
C
$INCLUDE:'WRDSIZ.INC'
C
C              -------------------------------------
C
      IS  = 1
      VAL = 1.0D0
C
      IF ( L.LT.0 ) GOTO 900
      IF ( M.GT.L ) GOTO 900
C
      IF ( L.EQ.0 ) GOTO 800
C
C
C         MINIMUM FACTOR MULTIPLY ONLY ALGORITHM
C
      PROD  =  1.0D0
C
      IF ( M.EQ.0 ) GOTO 140
C
      FLP  = L
      FLM  = L + 1
C
      DO  120  IM = 1, M
C
        FLP     = FLP + 1.0D0
        FLM     = FLM - 1.0D0
        PROD    = FLP *  FLM * PROD
C
120   CONTINUE
140   CONTINUE
C
      VAL    =  ( 2.0D0/ (2.0D0*L + 1.0 ) ) * PROD
C
C
C         GOOD CONSTANT
C
800   CONTINUE
C
      VAL = SQRT ( VAL )
      IS  = 0
C
900   CONTINUE
C
      PLMN  = VAL
      ISTAT = IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM Gravity NORMALIZATION CONST.'
      SUBROUTINE  PLMNGP ( L,M, PLMN, ISTAT )
C
C         GRAVITY   NORMALIZATION OF Plm  ( GEOPHYSICS CONVENTIONS )
C
C                  ==================================
C         =======  DIVIDE Plm BY THESE NORMALIZATIONS  =========
C                  ==================================
C
C         ********************************************
C         ***  FOR       Plm/Nor-GRAVITY          ****
C         ***  INTG(F*F)  =  2 for  M  =   0      ****
C         ***                4 for  M .NE. 0      ****
C         ********************************************
C
C              This normalization is such that the intergal
C              of the square of ( Plm * Cos(m*lamda) ) over
C              solid angle is 4*pi.  Here Cos(m*lamda) can also
C              be Sin (m*lamda).  Landa is longitude.
C
C              Due to the fact that only Cos/Sin ( m*Lamda ) are
C              involved in that intergal, and that avg. of Cos**2 =
C              average of Sin**2 = 1/2  get extra factor.  
C              BUT this average is 1 for m=0.  This gives Km factor.
C
C                                  ( L + M )!           1
C              Ngp **2 =         ------------- * ----------------
C                                  ( L - M )!     Km * ( 2*L + 1 )
C
C                   Where  Km  =   2  if  M .ne. 0
C                                  1  if  M  =   0
C
C              NOTICE THAT IN THE (2*L -1 )  THE 1 /2  IS MISSING
C
C    MODHIST:
C              2-2-94    CODED  JRC
C              2-12-94   (-1)**M PHASE VS RECURSIVE GENERATORS OF PLM
C                        ADDED
C              2-21-94   TOOK OUT -1**M
C
C
C    INPUT:
C              L         I*4       MAJOR INDEX
C              M         I*4       MINOR ( SECTORAL ) INDEX
C
C    OUTPUT:
C              PLMN      R*8       NORMALIZATION CONSTANT / SCHMIDT
C              ISTAT     I*4       0, OKAY, 1 INVALID INPUT
C
C
$INCLUDE:'WRDSIZ.INC'
C
C              -------------------------------------
C
      IS  = 1
      VAL = 1.0D0
C
      KM  = 2
      IF ( M.EQ.0 ) KM = 1
C
      IF ( L.LT.0 ) GOTO 900
      IF ( M.GT.L ) GOTO 900
C
C
      IF ( L.EQ.0 ) GOTO 800
C
C
C         MINIMUM FACTOR MULTIPLY ONLY ALGORITHM
C
      PROD  =  1.0D0
C
      IF ( M.EQ.0 ) GOTO 140
C
      FLP  = L
      FLM  = L + 1
C
      DO  120  IM = 1, M
C
        FLP     = FLP + 1.0D0
        FLM     = FLM - 1.0D0
        PROD    = FLP *  FLM * PROD
C
120   CONTINUE
140   CONTINUE
C
      VAL    =  PROD 
C
C         GOOD CONSTANT
C
800   CONTINUE
C
      VAL = VAL / ( KM * (2*L+1) )
      VAL = SQRT ( VAL )
      IS  = 0
C
900   CONTINUE
C
      PLMN  = VAL
      ISTAT = IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM Schmidt NORMALIZATION CONST.'
      SUBROUTINE  PLMNSC ( L,M, PLMN, ISTAT )
C
C         SCHNMDT   NORMALIZATION OF Plm  ( Mag. Field CONVENTIONS )
C
C                  ==================================
C         =======  DIVIDE Plm BY THESE NORMALIZATIONS  =========
C                  ==================================
C
C         ********************************************
C         ***  FOR       Plm/Nor-SCHMIDT          ****
C         ***  INTG(F*F)  =  2L+1 for  M  =   0   ****
C         ***                4L+2 for  M .NE. 0   ****
C         ********************************************
C
C              This normalization is such that the intergal
C              of the square of ( Plm * Cos(m*lamda) ) over
C              solid angle is 4pi/(2L+1).  Here Cos(m*lamda) 
C              can also be Sin (m*lamda).  Lamda is longitude.
C
C              Due to the fact that only Cos/Sin ( m*Lamda ) are
C              involved in that intergal, and that avg. of Cos**2 =
C              average of Sin**2 = 1/2  get extra factor.  
C              BUT this average is 1 for m=0.  This gives Km factor.
C
C                                  ( L + M )!           1
C      Nschmidt **2 =            ------------- * ----------------
C                                  ( L - M )!           Km 
C
C                   Where  Km  =   2  if  M .ne. 0
C                                  1  if  M  =   0
C
C              NOTICE THAT THE   (2*L -1 )   IS MISSING
C
C         ***  2-13-94  Phase:  via A.F. Handbook "Schmidt Functions"
C         ***                   don't have (-1)**m
C
C
C    MODHIST:
C              2-11-94    CODED  JRC FROM GRAVITY VERSION
C              2-13-94    PHASE-- ADDED (-1)**M
C              2-21-94   TOOK OUT -1**M
C
C
C    INPUT:
C              L         I*4       MAJOR INDEX
C              M         I*4       MINOR ( SECTORAL ) INDEX
C
C    OUTPUT:
C              PLMN      R*8       NORMALIZATION CONSTANT / SCHMIDT
C              ISTAT     I*4       0, OKAY, 1 INVALID INPUT
C
C
$INCLUDE:'WRDSIZ.INC'
C
C              -------------------------------------
C
      IS  = 1
      VAL = 1.0D0
C
      KM  = 2
      IF ( M.EQ.0 ) KM = 1
C
      IF ( L.LT.0 ) GOTO 900
      IF ( M.GT.L ) GOTO 900
C
C
      IF ( L.EQ.0 ) GOTO 800
C
C         MINIMUM FACTOR MULTIPLY ONLY ALGORITHM
C
      PROD  =  1.0D0
C
      IF ( M.EQ.0 ) GOTO 140
C
      FLP  = L
      FLM  = L + 1
C
      DO  120  IM = 1, M
C
        FLP     = FLP + 1.0D0
        FLM     = FLM - 1.0D0
        PROD    = FLP *  FLM * PROD
C
120   CONTINUE
140   CONTINUE
C
      VAL    =  PROD 
C
C         GOOD CONSTANT
C
800   CONTINUE
C
      VAL = VAL /  KM
      VAL = SQRT ( VAL )
      IS  = 0
C
900   CONTINUE
C
      PLMN  = VAL
      ISTAT = IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'COS/SIN (M*LONGITUDE)'
      SUBROUTINE X2MLA ( X, MMAX, CMLA, SMLA )
C
C             X(3)  POSITION TO M*LONGITUDE ARRAYS
C
C    MODHIST:
C              2-5-94    JRC CODED
C
C    INPUT:
C              X(3)      R*8       THREE VECTOR OF LOCATION
C                                  (UNITS NOT IMPORTANT, MAKES ANGLES)
C              MMAX      I*4       MAX M FOR ARRAYS
C
C    OUTPUT:
C              CMLA (MMAX+1)    R*8       COS ( M*LONGITUDE ) ARRAY
C              SMLA (MMAX+1)    R*8       SIN ( M*LONGITUDE ) ARRAY
C
C              DOES  COS/SIN ( 0 * LONGITUDE ) AS FIRST ELEMENT
C              ***   NOTE ARRAYS MMAX + 1 LONG !!   ***
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION   X(3), CMLA (*), SMLA (*)
C
C              --------------------------------------------
C
C
C              FIND LONGITUDE SIN / COS  FROM X,Y COMPONENTS
C
      XV   =   X(1)
      YV   =   X(2)
C
      RHO  =   SQRT ( XV**2 + YV**2 )
C
      IF ( RHO.GT. 1.0D-30 ) THEN
         CL  =  XV / RHO
         SL  =  YV / RHO
      ELSE
         CL   =    1.0D0
         SL   =    0.0D0
      ENDIF
C
      CMLA(1)   = 1.0D0
      SMLA(1)   = 0.0D0
C
      IF ( MMAX.LE.0 ) GOTO 900
C
      CMLL =  1.0D0
      SMLL =  0.0D0
C
      ITO  = 2
C
      DO   200  M = 1, MMAX
C
          CML  =  CL*CMLL - SL*SMLL
          SML  =  CL*SMLL + SL*CMLL
C
          CMLA ( ITO ) =  CML
          SMLA ( ITO ) =  SML
C
          ITO     =   ITO  + 1
          CMLL    =   CML
          SMLL    =   SML
C
200   CONTINUE
C
900   CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'COS/SIN (M*LONGITUDE)'
      SUBROUTINE LN2MLA ( FLON, MMAX, CMLA, SMLA )
C
C           LONGITUDE (DEG)  TO M*LONGITUDE ARRAYS
C
C    MODHIST:
C              2-5-94    JRC CODED
C
C    INPUT:
C              FLON      R*8       LONGITUDE IN DEG
C              MMAX      I*4       MAX M FOR ARRAYS
C
C    OUTPUT:
C              CMLA (MMAX+1)    R*8       COS ( M*LONGITUDE ) ARRAY
C              SMLA (MMAX+1)    R*8       SIN ( M*LONGITUDE ) ARRAY
C
C              DOES  COS/SIN ( 0 * LONGITUDE ) AS FIRST ELEMENT
C              ***   NOTE ARRAYS MMAX + 1 LONG !!   ***
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION    CMLA (*), SMLA (*)
C
C              --------------------------------------------
C
C
      DATA   PI    /   3.14159 26535 898D00 /
      DATA   TPI   /   6.28318 53071 796D0  /
      DATA   DTR   /   1.7453 292519943D-2  /
      DATA   RTD   /  57.29577 951308 2 D0  /
C
C
C              -------------------------------------------
C
      CL   =  COS ( DTR * FLON )
      SL   =  SIN ( DTR * FLON )
C
      CMLA(1)   = 1.0D0
      SMLA(1)   = 0.0D0
C
      IF ( MMAX.LE.0 ) GOTO 900
C
      CMLL =  1.0D0
      SMLL =  0.0D0
C
      ITO  = 2
C
      DO   200  M = 1, MMAX
C
          CML  =  CL*CMLL - SL*SMLL
          SML  =  CL*SMLL + SL*CMLL
C
          CMLA ( ITO ) =  CML
          SMLA ( ITO ) =  SML
C
          ITO     =   ITO  + 1
          CMLL    =   CML
          SMLL    =   SML
C
200   CONTINUE
C
900   CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM DERIV. VIA M+1'
      SUBROUTINE DXPLMM ( X, L,M, LMAX, PLMA, DPLM )
C
C         DERIVATIVE OF PLM(X)  
C            GIVEN X, PLM  ARRAY TO LMAX
C
C          m+1   ALGORITHM
C
C         Dx P[l,m] = -{ m * x * P[l,m] / (1 - x**2)  -
C                                P[l,m+1] / sqrt( 1 - x**2 ) }
C
C                   
C
C         MODHIST:
C                   2-8-94   JRC CODED
C                   2-21-94   TOOK OUT -1**M
C
C         INPUT:
C                   X         R*8       ARGUMENT
C                   L         I*4       L VALUE TO EVALUATE
C                   M         I*4       M VALUE TO EVALUATE
C
C                   LMAX      I*4       MAX L IN ARRAY 
C                   PLMA(.)   R*8       ARRAY OF PLM'S  AT SAME X
C                                       [0,0],[1,0],[1,1],[2,0]...ORDER
C                                       TO LMAX,LMAX 
C         OUTPUT
C                   DPLM      R*8       DERIVATIVE OF Plm  w.r.t X
C
C            NOT DEFINED AT X= +/- 1
C            NOT EVALUATED OUTSIDE (-1,1)
C                                  
C             NOT EVALUATED FOR:
C                                L > LMAX
C                                M > L
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  PLMA(*)
C
C              -------------------------------------------
C
      VAL = 0.0D0
      IF ( ABS ( X ).GE.1.0D0 ) GOTO 900
C
      IF ( LMAX.LT.0 ) GOTO 900
      IF ( L.GT.LMAX ) GOTO 900
      IF ( M.GT.L    ) GOTO 900
C
C            SPECIAL CASE  [0,0]
C
      IF ( L.EQ.0 ) GOTO 900
C
      SS   = 1.D0 - X**2
      S    = SQRT ( SS )
C
      INDX  =  L * ( L+1) /2    +  M  + 1
      PLM   =  PLMA ( INDX )
C
      IF ( M.LT.L ) THEN
        INDXP = INDX + 1
        PLMM  = PLMA ( INDXP )
      ELSE
        PLMM  =  0.0D0
      ENDIF
C
      VAL  = - M*X*PLM/SS  + PLMM / S
C
C
900   CONTINUE
C
      DPLM  = VAL
C
      RETURN
      END
$PAGE
$SUBTITLE:'PLM DERIV. VIA L-1'
      SUBROUTINE DXPLML ( X, L,M, LMAX, PLMA, DPLM )
C
C         DERIVATIVE OF PLM(X)  
C            GIVEN X, PLM  ARRAY TO LMAX
C
C          l-1   ALGORITHM
C
C         Dx P[l,m] = -{ l * x * P[l,m] / (1 - x**2)  +
C                      ( l+m ) * P[l+1,m] / ( 1 - x**2 ) }
C
C                   
C
C         MODHIST:
C                   2-8-94   JRC CODED
C
C         INPUT:
C                   X         R*8       ARGUMENT
C                   L         I*4       L VALUE TO EVALUATE
C                   M         I*4       M VALUE TO EVALUATE
C
C                   LMAX      I*4       MAX L IN ARRAY 
C                   PLMA(.)   R*8       ARRAY OF PLM'S  AT SAME X
C                                       [0,0],[1,0],[1,1],[2,0]...ORDER
C                                       TO LMAX,LMAX 
C         OUTPUT
C                   DPLM      R*8       DERIVATIVE OF Plm  w.r.t X
C
C            NOT DEFINED AT X= +/- 1
C            NOT EVALUATED OUTSIDE (-1,1)
C                                  
C             NOT EVALUATED FOR:
C                                L > LMAX
C                                M > L
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  PLMA(*)
C
C              -------------------------------------------
C
      VAL = 0.0D0
      IF ( ABS ( X ).GE.1.0D0 ) GOTO 900
C
      IF ( LMAX.LT.0 ) GOTO 900
      IF ( L.GT.LMAX ) GOTO 900
      IF ( M.GT.L    ) GOTO 900
C
C            SPECIAL CASE  [0,0]
C
      IF ( L.EQ.0 ) GOTO 900
C
      SS   = 1.D0 - X**2
C
      INDX  =  L * ( L+1) /2    +  M  + 1
      PLM   =  PLMA ( INDX )
C
      IF ( M.LT.L ) THEN
        INDXP = INDX - L
        PLML  = PLMA ( INDXP )
      ELSE
        PLML  =  0.0D0
      ENDIF
C
      VAL  = - (L*X*PLM  - (L+M)*PLML ) / SS
C
C
900   CONTINUE
C
      DPLM  = VAL
C
      RETURN
      END
$SUBTITLE:'SERIES SPHERICAL HAROMIC'
      SUBROUTINE  SUMSPH ( LMAX, A, FLATGD,FLON,ALTKM, CLM,SLM,
     $                     SUM, WORK )
C
C         EVALUATE A SPHERICAL HARMONIC / LEGENDRE.. SERIES
C
C    MODHIST:
C              2-25-94    JRC CODED
C
C    INPUT:
C              LMAX      I*4       MAX L ( OR N IN SOME NOTATIONS )
C              A         R*8       SCALE RADIUS VALUE
C              FLATGD    R*8       LATITUDE (GEODETIC) DEG
C              FLON      R*8       LONGITUDE
C              ALTKM     R*8       ALTITUDE IN KM
C
C
C              CLM (MX)  R*8       COSINE COEFFICIENT ARRAY
C              SLM (MX)  R*8       SIN       "         "
C                                  ( DIMENSION (LMAX+1)*(LMAX+2)/2 )
C    
C    OUTPUT:
C              SUM       R*8       SUM OF SERIES
C
C    USE:
C              WORK      R*8       WORK AREA  AT LEAST OF LENGTH
C                                  (LMAX+1)*(LMAX+2)/2  + 2*LMAX +2
C
C                                  ON EXIT HAS PLM'S FOLLOWED BY
C                                  COS(M*LAMDA), SIN(M*LAMDA) ARRAYS
C
C
C    EVALUATED:    ---  CALLS SERSPH
C
C              ------------------------------------------------------
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  CLM(*),SLM(*), WORK(*)
C
      DIMENSION  X(3)
C
C
C
      DATA    PI /    3.14159 26535 898D00 /
      DATA   TPI /    6.28318 53071 796D0  /
      DATA   DTR /    1.7453 292519943D-2  /
      DATA   RTD /   57.29577 951308 2 D0  /
C
C              WGS84 / GRS80 VALUES
C
      DATA    RE     /   6378.137D0            /
      DATA    ESETH  /   6.694 37999 013  D-3  /
C
C       -----------------------------------------------------------
C
C
C         BOMFORD / MORITZ ALGORITHM  LLH TO CARTESIAN
C         BASED ON:  LLHXYZ ( FLATGD, FLON, ALTKM, X )
C
C
      SLATGD  =   DSIN ( DTR * FLATGD )
      CLATGD  =   DCOS ( DTR * FLATGD )
      SLON    =   DSIN ( DTR * FLON )
      CLON    =   DCOS ( DTR * FLON )
C
      RMINOR  =   RE / DSQRT ( 1.0D0 - ESETH * SLATGD**2 )
C
      X(1)       =  ( RMINOR + ALTKM ) * CLATGD * CLON
      X(2)       =  ( RMINOR + ALTKM ) * CLATGD * SLON
      X(3)       =  ( (1.0D0 - ESETH ) * RMINOR  + ALTKM) * SLATGD
C
C
      CALL    SERSPH ( LMAX, A, X, CLM,SLM, SUM, WORK )
C
900   CONTINUE
C
      RETURN
C
      END
$SUBTITLE:'SERIES SPHERICAL HAROMIC'
      SUBROUTINE  GRDSPH ( LMAX, A, FLATGD,FLON,ALTKM, CLM,SLM,
     $                     GRAD, WORK )
C
C         EVALUATE GRADIENT OF A SPHERICAL HARMONIC SERIES
C
C    MODHIST:
C              2-25-94    JRC CODED
C
C    INPUT:
C              LMAX      I*4       MAX L ( OR N IN SOME NOTATIONS )
C              A         R*8       SCALE RADIUS VALUE
C              FLATGD    R*8       LATITUDE (GEODETIC) DEG
C              FLON      R*8       LONGITUDE
C              ALTKM     R*8       ALTITUDE IN KM
C
C
C              CLM (MX)  R*8       COSINE COEFFICIENT ARRAY
C              SLM (MX)  R*8       SIN       "         "
C                                  ( DIMENSION (LMAX+1)*(LMAX+2)/2 )
C    
C    OUTPUT:
C              GRAD(3)   R*8       E/N/U  GRADIENT
C                                      _________________
C                                  **** NOTE IS GEODETIC   *****
C                                      ------------------
C
C    USE:
C              WORK      R*8       WORK AREA  AT LEAST OF LENGTH
C                                  (LMAX+1)*(LMAX+2)/2  + 2*LMAX +2
C
C                                  ON EXIT HAS PLM'S FOLLOWED BY
C                                  COS(M*LAMDA), SIN(M*LAMDA) ARRAYS
C
C
C    EVALUATED:    ---  CALLS SRSPHG
C
C              ------------------------------------------------------
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  GRAD(3),CLM(*),SLM(*), WORK(*)
C
      DIMENSION  X(3)
C
C
C
      DATA    PI /    3.14159 26535 898D00 /
      DATA   TPI /    6.28318 53071 796D0  /
      DATA   DTR /    1.7453 292519943D-2  /
      DATA   RTD /   57.29577 951308 2 D0  /
C
C              WGS84 / GRS80 VALUES
C
      DATA    RE     /   6378.137D0            /
      DATA    ESETH  /   6.694 37999 013  D-3  /
C
C       -----------------------------------------------------------
C
C
C         BOMFORD / MORITZ ALGORITHM  LLH TO CARTESIAN
C         BASED ON:  LLHXYZ ( FLATGD, FLON, ALTKM, X )
C
C
      SLATGD  =   DSIN ( DTR * FLATGD )
      CLATGD  =   DCOS ( DTR * FLATGD )
      SLON    =   DSIN ( DTR * FLON )
      CLON    =   DCOS ( DTR * FLON )
C
      RMINOR  =   RE / DSQRT ( 1.0D0 - ESETH * SLATGD**2 )
C
      X(1)       =  ( RMINOR + ALTKM ) * CLATGD * CLON
      X(2)       =  ( RMINOR + ALTKM ) * CLATGD * SLON
      X(3)       =  ( (1.0D0 - ESETH ) * RMINOR  + ALTKM) * SLATGD
C
      CALL SRSPHG ( LMAX, A, X, CLM,SLM, GRAD, WORK )
C
C                 ********************************
C          *****   FOR GEODETIC IN, GEODETIC OUT   *****
C                 ********************************
C
      CALL ENUC2D ( FLATGD,ALTKM, GRAD, GRAD )
C
C
C
900   CONTINUE
C
      RETURN
C
      END
$SUBTITLE:'SERIES SPHERICAL HAROMIC'
      SUBROUTINE  SERSPH ( LMAX, A, X, CLM,SLM, SUM, WORK )
C
C         EVALUATE A SPHERICAL HARMONIC / LEGENDRE.. SERIES
C
C    MODHIST:
C              2-9-94    JRC CODED
C
C    INPUT:
C              LMAX      I*4       MAX L ( OR N IN SOME NOTATIONS )
C              A         R*8       SCALE RADIUS VALUE
C              X(3)      R*8       LOCATION OF EVALUATION
C
C              CLM (MX)  R*8       COSINE COEFFICIENT ARRAY
C              SLM (MX)  R*8       SIN       "         "
C                                  ( DIMENSION (LMAX+1)*(LMAX+2)/2 )
C    
C    OUTPUT:
C              SUM       R*8       SUM OF SERIES
C
C    USE:
C              WORK      R*8       WORK AREA  AT LEAST OF LENGTH
C                                  (LMAX+1)*(LMAX+2)/2  + 2*LMAX +2
C
C                                  ON EXIT HAS PLM'S FOLLOWED BY
C                                  COS(M*LAMDA), SIN(M*LAMDA) ARRAYS
C
C
C    EVALUATED:                ___
C                        Lmax |
C         SUM    =    Sum     |    (A/R)**(L+1)  *
C                         L=0 |___
c
C                             ____
C                          L |
C                      Sum   | P[L,M](x) * { CLM[L,M] * COS (M*Lamda) +
C                         M=0|____
C                            
C                                            SLM[L,M] * SIN (M*Lamda) }
C
C         Where:    x      =   cos ( Theta )
C                   Theta  =   co-latitude
C                   Lamda  =   longitude
C                   R      =   Magnitude ( X ) ,  X=input 3-vector
C
C              ------------------------------------------------------
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  X(3), CLM(*),SLM(*), WORK(*)
C
C
C
      DATA    PI /    3.14159 26535 898D00 /
      DATA   TPI /    6.28318 53071 796D0  /
      DATA   DTR /    1.7453 292519943D-2  /
      DATA   RTD /   57.29577 951308 2 D0  /
C
C       -----------------------------------------------------------
C
C 
      VAL  = 0.0D0
      IF ( LMAX.LT.0 ) GOTO 900
      IF ( A.LE.0.0D0) GOTO 900
C
C              COMPUTE THE ANGLES
C
      XV  =  X(1)
      YV  =  X(2)
      ZV  =  X(3)
C
      RS    =  XV**2 + YV**2 + ZV**2
C
C         IGNORE ORGIN COMPUTATIONS
C
      IF ( RS .LE.1.0D-60) GOTO 900
C
C
      R     =  DSQRT ( RS )
C
C         CT  = COS THETA
C
      IF( R .GT. 1.0D-30 )  THEN
          CT = ZV / R
      ELSE
          CT  = 1.0D0
          IF ( ZV.LT.0.0D0 ) CT = -1.0D0
      ENDIF
C
C         COMPUTE INDX BEG. PLM, CML, SML
C
      IXPLM   =  1
      IXCML   =  (LMAX+1)*(LMAX+2)/2  + IXPLM
      IXSML   =   LMAX + 1 + IXCML
C
C
      RRATIO  =   A / R
C
C              ---  COMPUTE VECTORS OF PLM, CML, SML  ---
C
      CALL PLMARY ( LMAX, CT, WORK(IXPLM), ISTP )
      IF ( ISTP.NE.0 ) GOTO 900
C
      CALL X2MLA ( X, LMAX, WORK(IXCML), WORK(IXSML) )
C
C              --------------------------------------------
C                     DOUBLE  SUMMATION BEGINS
C
      ARFACT  =  1.0D0
      IXP     =  IXPLM
C
      VAL     = 0.0D0
C
      DO   400 L = 0, LMAX
C
          ARFACT = ARFACT * RRATIO
          SUMM   = 0.0D0
C
          IXC   =  IXCML 
          IXS   =  IXSML 
C
          DO  300 M = 0, L
C
C
              PLMV  =  WORK ( IXP )
              CV    =  WORK ( IXC )
              SV    =  WORK ( IXS )
C
              TERM  =   PLMV * ( CV*CLM(IXP) + SV*SLM(IXP) )
              SUMM  =   SUMM + TERM
C
              IXP   = IXP + 1
              IXC   = IXC + 1
              IXS   = IXS + 1
C
300       CONTINUE
C
          VAL  =  ARFACT * SUMM  +  VAL
C
400   CONTINUE
C
C              --------------  END DOUBLE SUM  ------------
C
900   CONTINUE
C
      SUM  = VAL
C
      RETURN
      END
$SUBTITLE:'SERIES SPHER. HAR. GRADIENT'
      SUBROUTINE  SRSPHG ( LMAX, A, X, CLM,SLM, GRAD, WORK )
C
C         EVALUATE A SPHERICAL HARMONIC / LEGENDRE.. SERIES
C             -----------  GRADIENT  -----------
C                 RESULTS  E / N / UP  VECTOR
C
C
C    MODHIST:
C              2-16-94 JRC CODED
C              2-21-94 KILLED -1**M NORMAL.  MOD'S DERIV. RECURSION
C
C    INPUT:
C              LMAX      I*4       MAX L ( OR N IN SOME NOTATIONS )
C              A         R*8       SCALE RADIUS VALUE
C              X(3)      R*8       LOCATION OF EVALUATION
C
C              CLM (MX)  R*8       COSINE COEFFICIENT ARRAY
C              SLM (MX)  R*8       SIN       "         "
C                                  ( DIMENSION (LMAX+1)*(LMAX+2)/2 )
C    
C    OUTPUT:
C              GRAD(3)   R*8       GRADIENT  E,N,UP COMPONENTS
C                                       _________________
C                                  **** NOTE IS GEOCENTRIC *****
C                                      ------------------
C
C    USE:
C              WORK      R*8       WORK AREA  AT LEAST OF LENGTH
C                                  (LMAX+1)*(LMAX+2)/2  + 2*LMAX +2
C
C                                  ON EXIT HAS PLM'S FOLLOWED BY
C                                  COS(M*LAMDA), SIN(M*LAMDA) ARRAYS
C
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION  X(3),GRAD(3), CLM(*),SLM(*), WORK(*)
C
C
C
      DATA    PI /    3.14159 26535 898D00 /
      DATA   TPI /    6.28318 53071 796D0  /
      DATA   DTR /    1.7453 292519943D-2  /
      DATA   RTD /   57.29577 951308 2 D0  /
C
C       -----------------------------------------------------------
C
C 
      GN   = 0.0D0
      GE   = 0.0D0
      GU   = 0.0D0
C
      IF ( LMAX.LT.0 ) GOTO 900
      IF ( A.LE.0.0D0) GOTO 900
C
C              COMPUTE THE ANGLES
C
      XV  =  X(1)
      YV  =  X(2)
      ZV  =  X(3)
C
      RS    =  XV**2 + YV**2 + ZV**2
C
C         IGNORE ORGIN COMPUTATIONS
C
      IF ( RS .LE.1.0D-60) GOTO 900
C
C
      R     =  DSQRT ( RS )
C
C         CT  = COS THETA
C
      CT  = ZV / R
      STQ = 1.0D0 - CT**2
C
      IF( STQ .GT. 1.0D-60 )  THEN
          ST  = SQRT ( STQ )
      ELSE
C
C              POLAR LOCATIONS JUST WON'T WORK OUT
C                   DOING SOMETHING SIN(THETA)=1.0E-30
C
          CT  = 1.0D0
          STQ = 1.0D-60
          ST  = 1.0D-30
          IF ( ZV.LT.0.0D0 ) THEN
              CT = -1.0D0
              ST = - ST
          ENDIF
C
      ENDIF
C
C              MULTIPLIERS FOR TWO TERMS IN M+1 SCHEME OF DREV
C
      DRV1   =   CT    / STQ
      DRV2   =  -1.0D0 / ST
C
      RST    =   R * ST
C
C         COMPUTE INDX BEG. PLM, CML, SML
C
      IXPLM   =  1
      IXCML   =  (LMAX+1)*(LMAX+2)/2  + IXPLM
      IXSML   =   LMAX + 1 + IXCML
C
C
      RRATIO  =   A / R
C
C              ---  COMPUTE VECTORS OF PLM, CML, SML  ---
C
      CALL PLMARY ( LMAX, CT, WORK(IXPLM), ISTP )
      IF ( ISTP.NE.0 ) GOTO 900
C
      CALL X2MLA ( X, LMAX, WORK(IXCML), WORK(IXSML) )
C
C              --------------------------------------------
C                     DOUBLE  SUMMATION BEGINS
C
      ARFACT  =  1.0D0
      IXP     =  IXPLM
C
      VAL     = 0.0D0
C
      DO   400 L = 0, LMAX
C
          ARFACT = ARFACT * RRATIO
C
          IXC   =  IXCML 
          IXS   =  IXSML 
C
          TERMN = 0.0D0
          TERME = 0.0D0
          TERMU = 0.0D0
C
          DO  300 M = 0, L
C
              PLMV  =  WORK ( IXP )
              CV    =  WORK ( IXC )
              SV    =  WORK ( IXS )
C
              COFNOR =        CV * CLM(IXP)  +  SV * SLM(IXP)
              COFDRV = M * ( -SV * CLM(IXP)  +  CV * SLM(IXP) )
C
C             - SIN(THETA) * PLM DREV VALUE  FOR GRAD/NORTH
C
              PLMP   =   0.0D0
              IF  ( M.LT.L ) THEN
                  PLMP = WORK ( IXP + 1 )
              ENDIF
C
              DXPLM  =  - M *DRV1*PLMV - DRV2*PLMP
              TERMN  =   TERMN + ST*DXPLM * COFNOR
C
C                EAST TERM -- JUST M WEIGHT AND INVERT SIN, COS
C
              TERME  =  TERME +   PLMV *COFDRV
C
C                UP TERM  --- - (L+1) WEIGHT
C
              TERMU  =  TERMU - (L+1)*PLMV*COFNOR
C
              IXP   = IXP + 1
              IXC   = IXC + 1
              IXS   = IXS + 1
C
300       CONTINUE
C
C              NOW DO RANGE WEIGHTING AND OTHER M INVARIENT ITEMS
C
          GN  =  GN + TERMN * ARFACT / R
          GE  =  GE + TERME * ARFACT / ( RST )
          GU  =  GU + TERMU * ARFACT / R
C
400   CONTINUE
C
C              --------------  END DOUBLE SUM  ------------
C
900   CONTINUE
C
      GRAD(1)   =  GE
      GRAD(2)   =  GN
      GRAD(3)   =  GU
C
      RETURN
      END
