$TITLE: 'LINEAR STATISTICS ROUTINES'
$SUBTITLE:'STATL=INITIALIZE'
      SUBROUTINE STATL ( X, Y, VEC )
C-----
C             COMPUTE LINEAR STATISTICS
C
C       THREE ROUTINES ARE USED
C             STATL  --  CLEARS A VECTOR OF 10 ELEMENTS
C             STATXY --  INPUTS ONE POINT INTO THE VECTOR
C             STATR  --  GET RESULTS INTO STANDARD FORM
C                           FOR PRINTING BY BY PLSTAT
C
C    VEC INDEX   OUTPUT             USE DURING COMPUTATION
C       1         N                  N
C       2         XBAR               SUM X
C       3         YBAR               SUM Y
C       4         SIGMA X            SUM X**2
C       5         SIGMA Y            SUM Y**2
C       6         A (Y=A+B*X)        SUM X*Y
C       7         B
C       8         RHO  (REG COEF)
C       9         SIGMA Y X  (COND. SIGMA)
C      10         SIGMA B
C
C
C          5 JUNE 1987   /SAC/   CONVERTED TO FORTRAN77 ON MICRO
C         13 AUG  1986   /RSM/   CONVERTED TO FORTRAN V
C-----
$INCLUDE:'WRDSIZ.INC'

      DIMENSION  VEC(10)

      CALL SETVAL ( VEC, 10, 0.D0 )

      RETURN
      END
$PAGE
$SUBTITLE:'COMPUTE RESULTS'
      SUBROUTINE STATR ( X, Y, VEC )
C-----
C             COMPUTE LINEAR STATISTICS
C
C       THREE ROUTINES ARE USED
C             STATL  --  CLEARS A VECTOR OF 10 ELEMENTS
C             STATXY --  INPUTS ONE POINT INTO THE VECTOR
C             STATR  --  GET RESULTS INTO STANDARD FORM
C                           FOR PRINTING BY BY PLSTAT
C
C    VEC INDEX   OUTPUT             USE DURING COMPUTATION
C       1         N                  N
C       2         XBAR               SUM X
C       3         YBAR               SUM Y
C       4         SIGMA X            SUM X**2
C       5         SIGMA Y            SUM Y**2
C       6         A (Y=A+B*X)        SUM X*Y
C       7         B
C       8         RHO  (REG COEF)
C       9         SIGMA Y X  (COND. SIGMA)
C      10         SIGMA B
C
C
C          5 JUNE 1987   /SAC/   CONVERTED TO FORTRAN77 ON MICRO
C         13 AUG  1986   /RSM/   CONVERTED TO FORTRAN V
C-----
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION  VEC(10)
C
      N = VEC(1)
      FN = VEC(1)
      SX = VEC(2)
      SY = VEC(3)
      SXX = VEC(4)
      SYY = VEC(5)
      SXY = VEC(6)
C
      CALL SETVAL ( VEC, 10, 0.0D0 )
C
      IF ( N .LE. 0 )   GOTO 900
C
      FNM1 = FN - 1.0D0
      XBAR = SX / FN
      YBAR = SY / FN
      VEC(1) = FN
      VEC(2) = XBAR
      VEC(3) = YBAR
C
      IF ( N .LE. 1 )   GOTO 900
C
C        ---   COMPUTE STATISTICS   ---
C
      SGXS = ( SXX - ( XBAR * XBAR * FN ) ) / FNM1
      SGYS = ( SYY - ( YBAR * YBAR * FN ) ) / FNM1
C
      SGX = DSQRT( DABS( SGXS ) )
      SGY = DSQRT( DABS( SGYS ) )
C
      RN = SXY - ( XBAR * YBAR * FN )
      R = 0.0D0
      TESTB = FNM1 * SGX * SGY
      IF ( TESTB .NE. 0.0D0 )     R = RN / TESTB
C
      BI = 0.0D0
      TESTA = SXX - ( XBAR * XBAR * FN )
      IF ( TESTA .NE. 0.0D0 )     BI = RN / TESTA
C
      AI = YBAR - ( BI * XBAR )
C
      VEC(4) = SGX
      VEC(5) = SGY
      VEC(6) = AI
      VEC(7) = BI
      VEC(8) = R
C
      IF ( N .LE. 2 )    GOTO 900
C
      SGYXS = ( FNM1 * SGYS * ( 1.0D0 - ( R * R ))) / ( FN - 2.0D0 )
      SGYX = DSQRT( DABS( SGYXS ) )
C
      SGB = 0.0D0
      TESTV = SGX * DSQRT( DABS( FNM1 ) )
      IF ( TESTV .NE. 0.0D0 )    SGB = SGYX / TESTV
C
      VEC(9) = SGYX
      VEC(10) = SGB
C
C             ---  RETURN  ---
C
  900 CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'INPUT X,Y PAIR'
      SUBROUTINE STATXY ( X, Y, VEC )
C-----
C             COMPUTE LINEAR STATISTICS
C
C       THREE ROUTINES ARE USED
C             STATL  --  CLEARS A VECTOR OF 10 ELEMENTS
C             STATXY --  INPUTS ONE POINT INTO THE VECTOR
C             STATR  --  GET RESULTS INTO STANDARD FORM
C                           FOR PRINTING BY BY PLSTAT
C
C    VEC INDEX   OUTPUT             USE DURING COMPUTATION
C       1         N                  N
C       2         XBAR               SUM X
C       3         YBAR               SUM Y
C       4         SIGMA X            SUM X**2
C       5         SIGMA Y            SUM Y**2
C       6         A (Y=A+B*X)        SUM X*Y
C       7         B
C       8         RHO  (REG COEF)
C       9         SIGMA Y X  (COND. SIGMA)
C      10         SIGMA B
C
C
C          5 JUNE 1987   /SAC/   CONVERTED TO FORTRAN77 ON MICRO
C         13 AUG  1986   /RSM/   CONVERTED TO FORTRAN V
C-----
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION  VEC(10)
C
      XI = X
      YI = Y
      VEC(1) = VEC(1) + 1.0D0
      VEC(2) = VEC(2) + XI
      VEC(3) = VEC(3) + YI
      VEC(4) = VEC(4) + ( X * X )
      VEC(5) = VEC(5) + ( Y * Y )
      VEC(6) = VEC(6) + ( X * Y )
C
      RETURN
      END

