      SUBROUTINE SVDUWV  (A,MI,NI,MPI,NPI,W,V)
C
C      SINGULAR VALUE DECOMPOSITION OF MARTIX INTO
C      U * W * VT
C
C      BASED ON NUM. REC.  SVDCMP ROUTINE
C      NUMERICAL RECIPES PG 52+
C      WHICH IS BASED ON FORSYTHE, AND  LINPACK, AND ...
C
C       A    =   MATRIX  PHYSICAL  MP X NP
C                        LOGICAL USE M X N
C
C      DECOMPOSED INTO THREE MATRIX FACTORS
C            A = U * W * VT
C
C        U  =  M X N  ( IN MP X NP HERE )  
C              COLUMN ORTHOGNORMAL
C               IE  SUM ( U(I,K) * U( J,K ) OVER I  = DELTA(I,J)
C                   ( DELTA(I,J) = KRONKER DELTA = 1 IF I=J, 0 ELSE )
C
C        W  =  N X N DIAGNAL MATRIX   
C        VT =  V TRANSPOSED
C        V  =  N X N   ORTHONORMAL MATRIX  ( VT = V INVERSE )
C
C       HERE:
C            U   RETURNED IN A
C            W   RETURNED JUST AS VECTOR -- IE JUST DIAGNAL ELEMENTS
C            V   RETURNED
C
C   ---    JRC / ARL:UT  ---  TRANSFERED TO MICRO
C                             MADE ALL REALS REAL*8
C                             EXTERNAL INTEGERS I*4
C                             INTERNAL INTEGERS I*2
C                             ( SPECIAL CALL VAR FOR N,M,NP,MP)
C                             SET MAX DIMENSION DOWN TO 15              
C
      IMPLICIT REAL*8   ( A-H, O-Z)
      IMPLICIT INTEGER*2 ( I-N )
C
C    TYPES FOR CALL ARGUMENTS 
C
      REAL*8   A, W, V
      INTEGER*4  NI,MI,NPI,MPI
C
      PARAMETER (NMAX=15)
      DIMENSION A(MPI,NPI),W(NPI),V(NPI,NPI),RV1(NMAX)
C
      N   =  NI
      M   =  MI
      MP  =  MPI
      NP  =  NPI
C
C   --------------------------------------------------------
C      HOUSEHOLDER REDUCTION TO BI-DIAGNONAL FORM
C   --------------------------------------------------------
C
      G=0.0D0
      SCALE=0.0D0
      ANORM=0.0D0
C
      DO 25 I=1,N
        L=I+1
        RV1(I)=SCALE*G
        G=0.0D0
        S=0.0D0
        SCALE=0.0D0
C
        IF (I.LE.M) THEN
C
          DO 11 K=I,M
            SCALE=SCALE+DABS(A(K,I))
11        CONTINUE
C
          IF (SCALE.NE.0.0D0) THEN
C 
           DO 12 K=I,M
              A(K,I)=A(K,I)/SCALE
              S=S+A(K,I)*A(K,I)
12          CONTINUE
C
            F=A(I,I)
            G=-DSIGN(DSQRT(S),F)
            H=F*G-S
            A(I,I)=F-G
            IF (I.NE.N) THEN
C 
             DO 15 J=L,N
                S=0.0D0
C
                DO 13 K=I,M
                  S=S+A(K,I)*A(K,J)
13              CONTINUE
C
                F=S/H
                DO 14 K=I,M
                  A(K,J)=A(K,J)+F*A(K,I)
14              CONTINUE
15            CONTINUE
C
            ENDIF
C
            DO 16 K= I,M
              A(K,I)=SCALE*A(K,I)
16          CONTINUE
C
          ENDIF
        ENDIF
C
        W(I)=SCALE *G
        G=0.0D0
        S=0.0D0
        SCALE=0.0D0
C
        IF ((I.LE.M).AND.(I.NE.N)) THEN
          DO 17 K=L,N
            SCALE=SCALE+DABS(A(I,K))
17        CONTINUE
C
          IF (SCALE.NE.0.0D0) THEN
C
            DO 18 K=L,N
              A(I,K)=A(I,K)/SCALE
              S=S+A(I,K)*A(I,K)
18          CONTINUE
C
            F=A(I,L)
            G=-DSIGN(DSQRT(S),F)
            H=F*G-S
            A(I,L)=F-G
C
            DO 19 K=L,N
              RV1(K)=A(I,K)/H
19          CONTINUE
C
            IF (I.NE.M) THEN
              DO 23 J=L,M
                S=0.0D0
C
                DO 21 K=L,N
                  S=S+A(J,K)*A(I,K)
21              CONTINUE
C
                DO 22 K=L,N
                  A(J,K)=A(J,K)+S*RV1(K)
22              CONTINUE
C
23            CONTINUE
C
            ENDIF
C
            DO 24 K=L,N
              A(I,K)=SCALE*A(I,K)
24          CONTINUE
C
          ENDIF
C
        ENDIF
C
        ANORM=DMAX1(ANORM,(DABS(W(I))+DABS(RV1(I))))
25    CONTINUE
C
C   -----------------------------------------------------
C        ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS
C   -----------------------------------------------------
C
      DO 32 I=N,1,-1
C
        IF (I.LT.N) THEN
          IF (G.NE.0.0D0) THEN
C
            DO 26 J=L,N
C
C                             (STAGED DIVIDE TO AVOID UNDERFLOW)
C
              V(J,I)=(A(I,J)/A(I,L))/G
26          CONTINUE
C
            DO 29 J=L,N
              S=0.0D0
C
              DO 27 K=L,N
                S=S+A(I,K)*V(K,J)
27            CONTINUE
C
              DO 28 K=L,N
                V(K,J)=V(K,J)+S*V(K,I)
28            CONTINUE
C
29          CONTINUE
C
          ENDIF
C
          DO 31 J=L,N
            V(I,J)=0.0D0
            V(J,I)=0.0D0
31        CONTINUE
C
        ENDIF
C
C
        V(I,I)=1.0D0
        G=RV1(I)
        L=I
32    CONTINUE
C
C          ---------------------------------------------------
C          ACCUMULATION OF LEFT-HAND TRANSFORMATIONS
C          ---------------------------------------------------
C
      DO 39 I=N,1,-1
C
        L=I+1
        G=W(I)
        IF (I.LT.N) THEN
C
          DO 33 J=L,N
            A(I,J)=0.0D0
33        CONTINUE
C
        ENDIF
        IF (G.NE.0.0D0) THEN
          G=1.0D0/G
          IF (I.NE.N) THEN
C
            DO 36 J=L,N
              S=0.0D0
C
              DO 34 K=L,M
                S=S+A(K,I)*A(K,J)
34            CONTINUE
C
              F=(S/A(I,I))*G
C
              DO 35 K=I,M
                A(K,J)=A(K,J)+F*A(K,I)
35            CONTINUE
C
36          CONTINUE
C
          ENDIF
C
C
          DO 37 J=I,M
            A(J,I)=A(J,I)*G
37        CONTINUE
C
        ELSE
C
C 
          DO 38 J= I,M
            A(J,I)=0.0D0
38        CONTINUE
C
        ENDIF
C
C
        A(I,I)=A(I,I)+1.0D0
39    CONTINUE
C
C       -----------------------------------------------------------
C       DIAGONALIZATION OF THE BI-DIAGONAL FORM
C       -----------------------------------------------------------
C
C        LOOP OVER SINGULAR VALUES
C          LOOP OVER MAX 30 ITERATIONS
C            TEST FOR SPLITTINGS
C
C    --NOTE  RV1(1) = 0.0 BY DESIGN
C
C
      DO 49 K=N,1,-1
C
        DO 48 ITS=1,30
C
          DO 41 L=K,1,-1
            NM=L-1
            IF ((DABS(RV1(L))+ANORM).EQ.ANORM)  GO TO 2
            IF ((DABS(W(NM))+ANORM).EQ.ANORM)  GO TO 1
41        CONTINUE
C
C
   1      C=0.0D0
          S=1.0D0
C
          DO 43 I=L,K
            F=S*RV1(I)
            IF ((DABS(F)+ANORM).NE.ANORM) THEN
              G=W(I)
              H=DSQRT(F*F+G*G)
              W(I)=H
              H=1.0D0/H
              C= (G*H)
              S=-(F*H)
C
              DO 42 J=1,M
                Y=A(J,NM)
                Z=A(J,I)
                A(J,NM)=(Y*C)+(Z*S)
                A(J,I)=-(Y*S)+(Z*C)
42            CONTINUE
C
            ENDIF
C
43        CONTINUE
C
C      ----   CONVERGENCE   ----
C
   2       Z=W(K)
C
          IF (L.EQ.K) THEN
            IF (Z.LT.0.0D0) THEN
              W(K)=-Z
C
              DO 44 J=1,N
                V(J,K)=-V(J,K)
44            CONTINUE
C
            ENDIF
C
C
            GO TO 3
C 
         ENDIF
C
C
          IF (ITS.EQ.30) PAUSE 'SVD No convergence in 30 iterations'
C
          X=W(L)
          NM=K-1
          Y=W(NM)
          G=RV1(NM)
          H=RV1(K)
          F=((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
          G=DSQRT(F*F+1.0D0)
          F=((X-Z)*(X+Z)+H*((Y/(F+DSIGN(G,F)))-H))/X
C
C     -------------------------------------------------------------
C      Q-R  TRANSFORMATION
C     --------------------------------------------------------------
C
          C=1.0D0
          S=1.0D0
C
          DO 47 J=L,NM
            I=J+1
            G=RV1(I)
            Y=W(I)
            H=S*G
            G=C*G
            Z=DSQRT(F*F+H*H)
            RV1(J)=Z
            C=F/Z
            S=H/Z
            F= (X*C)+(G*S)
            G=-(X*S)+(G*C)
            H=Y*S
            Y=Y*C
C
            DO 45 NM=1,N
              X=V(NM,J)
              Z=V(NM,I)
              V(NM,J)= (X*C)+(Z*S)
              V(NM,I)=-(X*S)+(Z*C)
45          CONTINUE
C
            Z=DSQRT(F*F+H*H)
            W(J)=Z
C
            IF (Z.NE.0.0D0) THEN
              Z=1.0D0/Z
              C=F*Z
              S=H*Z
            ENDIF
C
            F= (C*G)+(S*Y)
            X=-(S*G)+(C*Y)
C
            DO 46 NM=1,M
              Y=A(NM,J)
              Z=A(NM,I)
              A(NM,J)= (Y*C)+(Z*S)
              A(NM,I)=-(Y*S)+(Z*C)
46          CONTINUE
C
47        CONTINUE
C
          RV1(L)=0.0D0
          RV1(K)=F
          W(K)=X
48      CONTINUE
C
C
   3    CONTINUE
C
C
49    CONTINUE
C
C
      RETURN
      END
