$TITLE:'SMOOTH-WINDOWS'
$SUBTITLE:'INIT-WINDOW-SMOOTHER'
      SUBROUTINE WINDWI ( XZ,DX, ITYPE, WIDTH, WINDWP, ISTAT )
C
C        INITIALIZE WINDOW PARMETER VECTOR
C
C    INPUT:
C              XZ         XZERO -- FOR EPOCH OF OUTPUT DEFINATION
C              DX         DIF IN X OF OUTPUT VALUES
C                         OUTPUTS AT  XZ +  N * DX,  N= INTEGER
C              ITYPE      TYPE WINDOW
C                         1  =    SQUARE
C                         2  =    TRIANGLE
C                         3  =    COSINE
C                         4  =    RAISED COS ( HANN )
C                         5  =    COS ON PEDISTAL ( HAMMING )
C
C             WIDTH       FULL WIDTH OF WINDOW  IN X UNITS
C                         WINDOW ZERO OUTSIDE ( -WIDTH/2, WIDTH/2)
C                         ABOUT POINTS  XZ + N * DX
C
C    OUTPUT:
C             WINDWP     VECTOR OF PARAMETERS   5 LONG
C                    1   XZ
C                    2   DX
C                    3   FLOAT ( ITYPE )
C                    4   WIDTH
C                    5   NUMBER OF POINTS IN USE SCRATCH ARRAYS
C                        ( WIDTH/DX )  + 1
C
C            ISTAT       ERROR/STATUS  0= OKAY, OTHER = PROBLEM
C
C
C        NOTE:  IF SEVERAL PARAMETERS TREATED IDENTICALY CAN HAVE
C               ONE PARAMETER VECTOR --  !!!  BUT EACH HAS OWN 
C               SCRATCH ARRAY IN WINDOW ROUTINE  !!!
C
C        SMOOTHER WINDOW MODELS
C
C             NORMALIZED  X,   XN  = ( X - XPOINT ) / WIDTH
C
C             YOUT  =   SUM( WEIGHT(XN) * Y) /  SUM ( WEIGHT(XN) )
C
C             SUMMATION OVER ALL POINTS IN (-0.5,0.5 ), IF NO POINTS
C             WILL BE NOW OUTPUT FOR THAT POINT  ( JUMP SEVERAL POINT
C             AND OUTPUT ALL CURRENT POINTS WHEN BIG JUMP IN INPUT
C             X VALUE )
C
C      WEIGHTS    W(XN)  ,   NORMALIZED SO 1.0 AT CENTER, XN=0.0
C
C      TYPE 1  SQUARE    W(XN) =  1.0
C           2  TRIANGLE  W(XN) =  1.0 - 2.0*ABS(XN)
C           3  COSINE    W(XN) =  COS( PI * XN )
C           4  HANN      W(XN) =  ( 1.0 + COS (2*PI * XN ) ) / 2.0
C           5  HAMMING   W(XN) =  ( 0.54 + 0.46*COS(2*PI*XN) )
C
C             
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION  WINDWP ( 5 )
C
C     ---------------------------------------------------------
C
C
      IS         = 0
      WINDWP(1)  =  XZ
C
      DXU =         ABS ( DX )
C
      IF ( DX.LE.0.0D0 ) THEN
         IF ( DXU.EQ. 0.0D0 ) DXU = 1.0D0
         IS = IS - 1
      ENDIF
C
      WINDWP(2)   = DXU
C
      IT  =   ITYPE
      IF ( IT.LT.1  .OR.  IT.GT.5 )  THEN
            IT = 1
            IS =  IS - 10
      ENDIF

      WINDWP(3)  = IT
C
      W  =  WIDTH
      IF ( W.LE.0.0D0 ) THEN
           IS = IS - 100
           W = 1.0D0
      ENDIF
C
      WINDWP(4)  = W
C
      NSAV  =  W / DXU + 1.00
      WINDWP(5) = NSAV
C
900   CONTINUE
C
      ISTAT = IS
C
      RETURN
      END
$PAGE
$SUBTITLE:'WINDOW-SMOOTHER DATA'
      SUBROUTINE  WINDWS  ( X, Y, WINDWP, NOUT, XYOUT, WORK )
C
C         INPUT DATA/GET OUTPUT FROM WINDOW SMOOTHER
C      
C    INPUT:
C
C         X         CURRENT  X INPUT VALUE
C         Y         CURRENT  Y INPUT VALUE
C
C      !!!    DATA MUST BE IN INCREASING ORDER OF X  !!!
C
C         WINDWP    PARAMTER VECTOR -- SETUP  BY WINDWI, 5 LONG
C                    (5) = NUMBER OF ITEMS IN WORK AREA ( NSAVE )
C                          AND POTENTIAL OUTPUTS
C
C    OUTPUT:
C
C         NOUT         NUMBER OF VALUES RETURNED ON CURRENT CALL
C                      0 TO WINDWP(5) ( NSAVE )
C
C         XYOUT(3,N)   OUTPUT DATA IF ANY
C              (1,.)   X VALUE OF SMOOTHED POINT
C              (2,.)   Y VALUE OF SMOOTHED POINT
C              (3,.)   NUMBER OF POINTS
C
C         WORK (4 * ( NSAVE+1) )  =  WORK AREA 
C                                    MUST BE 4 + 4*NSAVE LONG AT LEAST
C
C
C      !!!! NOTE BENE  !!!
C           WORK(1,1)  MUST BE SET TO ZERO FOR INITIALIZATION
C
C       USE OF WORK
C                   (1,1)   =    INDEX OF FIRST DATA POINT
C                   (2,1)   =    NUMBER OF DATA POINTS = WINDWP(5)
C                   (3,1)        UNUSED
C                   (4,1)        UNUSED
C
C                   GENERAL
C                   (1,.)        X   OF POINT TO OUTPUT
C                   (2,.)        SUM ( W*Y )  FOR POINT
C                   (3,.)        SUM ( W )    FOR POINT
C                   (4,.)        NUMBER OF POINTS CONTRIBUTING
C
C        (.2) ON ARE USED AS A CIRCULAR BUFFER OF POINTS
C        WITH (1,1)  BEING THE INDEX INTO THE FIRST ONE
C        -- IF IT IS .LE. 0  THEN INITIALIZE ---
C
C 
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION   XYOUT(3,*), WORK(4,*), WINDWP(5)
C
C
      DATA   PI / 3.14159 26535 8979 D0 /
C              
C-----------------------------------------------------------------
C
C
C
      XZ    = WINDWP(1)
      DX    = WINDWP(2)
      ITY   = WINDWP(3)
      W     = WINDWP(4)
      NSAVE = WINDWP(5)
C
      TPI   =  2.0D0 * PI
C
C          FIND FIRST POINT X VALUE CURRENT POINT INSIDE WINDOW
C
      XB      =  X - W/2.0D0
      XBN     =  XB / DX
      IXBN    =  XBN
      IF ( XBN .LT. 0.0D0 ) IXBN = IXBN -1
C
      IXBN     =  IXBN + 1
      XPBEG    =  IXBN * DX
C
C
      IXB    =  WORK(1,1 )
      IF ( IXB.GT.0 ) GOTO  200
C
C         -----   INITIAL  CALL -- SET UP POINTS   ----
C    ---------------------------------------------------------------
C
      WORK(1,1)   =  2.0D0
      WORK(2,1)   =  NSAVE
      WORK(3,1)   =  0.0D0
      WORK(4,1)   =  0.0D0
C
      XNOW     =  XPBEG
      DO    100  I = 1, NSAVE
C
          IX  = I  + 1
C
          WORK(1,IX)  =   XNOW
          WORK(2,IX)  =   0.0D0
          WORK(3,IX)  =   0.0D0
          WORK(4,IX)  =   0.0D0
C
          XNOW        =   XNOW + DX
C
100   CONTINUE
C
C        ---------------------------------------------------------
C
C        --------  NORMAL WORK --  ADD TO POINTS AS NEEDED  -------
C
200   CONTINUE
C
      IXBEG  =   WORK(1,1)
C
      IXOUT  =   0
      NNEW   =   0
C
      IX    = IXBEG - 1
      NTOP  = NSAVE + 1
      DO  400  I = 1, NSAVE
C
          IX  = IX + 1
          IF ( IX.GT.NTOP ) IX = 2
C
          XN  =  ( X - WORK(1,IX) ) / W
          IF ( XN .GT. 0.5D0 ) THEN
C 
C              PAST POINT  -- OUTPUT IF HAS DATA
C
              N  = WORK(4,IX)
              SW = WORK(3,IX)
              IF ( N.GT.0 .AND. SW.GT.0.0D0 ) THEN
C
                  SWY  =  WORK(2,IX)
                  YOUT =  SWY/ SW
                  IXOUT = IXOUT + 1
                  XYOUT(1,IXOUT)  =  WORK ( 1,IX )
                  XYOUT(2,IXOUT)  =  YOUT
                  XYOUT(3,IXOUT)  =  N
C
              ENDIF
C
C           NEW POINT --  ACCOUNT FOR JUMP IF NX POINT NOT USEFUL
C
              NNEW    =  NNEW + 1
              IF ( NNEW.EQ.1 ) THEN
C
                   XNEW    = WORK(1,IX) + NSAVE * DX
                   XN      =  ( X - XNEW ) / W
                   IF ( XN .GT. 0.5D0 )  XNEW = XPBEG
C
              ELSE
                   XNEW    = XNEW + DX
              ENDIF
C
              XN      =  ( X - XNEW ) / W
C
              WORK(1,IX)  = XNEW
              WORK(2,IX)  = 0.0D0
              WORK(3,IX)  = 0.0D0
              WORK(4,IX)  = 0.0D0
          ENDIF
C
          IF ( XN .GT. 0.5D0  .OR.  XN .LT. -0.5D0 ) GOTO 400
C
C                 ADD POINT INTO SUMS
C
C            COMPUTE WEIGHT
C
          WT  = 1.0D0
          IF ( ITY .EQ. 2 )   WT =  1.0D0 - 2.0D0 * DABS( XN )
          IF ( ITY .EQ. 3 )   WT =  DCOS ( PI * XN )
          IF ( ITY .EQ. 4 )   WT =  0.5D0 * (1.0D0 + DCOS(TPI*XN)) 
          IF ( ITY .EQ. 5 )   WT =   0.54D0 + 0.46D0*DCOS(TPI*XN)
C
C             ADD IN
C
          WORK(2,IX)  =  WORK(2,IX)  +  WT*Y
          WORK(3,IX)  =  WORK(3,IX)  +  WT
          WORK(4,IX)  =  WORK(4,IX)  +  1.0D0
C
C
400   CONTINUE
C
C       FIND NEW BEG INDEX IF CHANGED
C
      IF ( NNEW.NE.0 ) THEN
C
         IXB  = 2
         XB   = WORK ( 1,2 )
C
         DO 600 I= 1, NSAVE
            IX = I + 1
            XN = WORK ( 1,IX)
            IF ( XN.LT.XB ) THEN
               XB = XN
               IXB= IX
            ENDIF
600      CONTINUE
C
         WORK(1,1)  = IXB
C
      ENDIF
C
C
900   CONTINUE
C
      NOUT  = IXOUT
C
      RETURN
      END
